@@ -33,8 +33,8 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
  * - back-port this new code into tools/winedump/msmangle.c
  */
 
-/* How data types modifiers are stored:
- * M (in the following definitions) is defined for 
+/* How data types qualifiers are stored:
+ * M (in the following definitions) is defined for
  * 'A', 'B', 'C' and 'D' as follows
  *      {<A>}:  ""
  *      {<B>}:  "const "
@@ -77,14 +77,22 @@ struct parsed_symbol
     unsigned            avail_in_first; /* number of available bytes in head block */
 };
 
+enum datatype_e
+{
+    DT_NO_LEADING_WS = 0x01,
+    DT_NO_LRSEP_WS = 0x02,
+};
+
 /* Type for parsing mangled types */
 struct datatype_t
 {
     const char*         left;
     const char*         right;
+    enum datatype_e     flags;
 };
 
 static BOOL symbol_demangle(struct parsed_symbol* sym);
+static char* get_class_name(struct parsed_symbol* sym);
 
 /******************************************************************
  *		und_alloc
@@ -285,9 +293,15 @@ static char* WINAPIV str_printf(struct parsed_symbol* sym, const char* format, .
     return tmp;
 }
 
+enum datatype_flags
+{
+    IN_ARGS = 0x01,
+    WS_AFTER_QUAL_IF = 0x02,
+};
+
 /* forward declaration */
 static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
-                              struct array* pmt, BOOL in_args);
+                              struct array* pmt, enum datatype_flags flags);
 
 static const char* get_number(struct parsed_symbol* sym)
 {
@@ -361,7 +375,7 @@ static char* get_args(struct parsed_symbol* sym, struct array* pmt_ref, BOOL z_t
             sym->current++;
             break;
         }
-        if (!demangle_datatype(sym, &ct, pmt_ref, TRUE))
+        if (!demangle_datatype(sym, &ct, pmt_ref, IN_ARGS))
             return NULL;
         /* 'void' terminates an argument list in a function */
         if (z_term && !strcmp(ct.left, "void")) break;
@@ -394,70 +408,137 @@ static char* get_args(struct parsed_symbol* sym, struct array* pmt_ref, BOOL z_t
     return args_str;
 }
 
-/******************************************************************
- *		get_modifier
- * Parses the type modifier. Always returns static strings.
- */
-static BOOL get_modifier(struct parsed_symbol *sym, const char **ret, const char **ptr_modif)
+static void append_extended_qualifier(struct parsed_symbol *sym, const char **where,
+                                      const char *str, BOOL is_ms_keyword)
 {
-    *ptr_modif = NULL;
-    if (*sym->current == 'E')
+    if (!is_ms_keyword || !(sym->flags & UNDNAME_NO_MS_KEYWORDS))
     {
-        if (!(sym->flags & UNDNAME_NO_MS_KEYWORDS))
+        if (is_ms_keyword && (sym->flags & UNDNAME_NO_LEADING_UNDERSCORES))
+            str += 2;
+        *where = *where ? str_printf(sym, "%s%s%s%s", *where, is_ms_keyword ? " " : "", str, is_ms_keyword ? "" : " ") :
+            str_printf(sym, "%s%s", str, is_ms_keyword ? "" : " ");
+    }
+}
+
+static void get_extended_qualifier(struct parsed_symbol *sym, struct datatype_t *xdt)
+{
+    unsigned fl = 0;
+    xdt->left = xdt->right = NULL;
+    xdt->flags = 0;
+    for (;;)
+    {
+        switch (*sym->current)
         {
-            *ptr_modif = "__ptr64";
-            if (sym->flags & UNDNAME_NO_LEADING_UNDERSCORES)
-                *ptr_modif = *ptr_modif + 2;
+        case 'E': append_extended_qualifier(sym, &xdt->right, "__ptr64", TRUE);     fl |= 2; break;
+        case 'F': append_extended_qualifier(sym, &xdt->left,  "__unaligned", TRUE); fl |= 2; break;
+#ifdef _UCRT
+        case 'G': append_extended_qualifier(sym, &xdt->right, "&", FALSE);          fl |= 1; break;
+        case 'H': append_extended_qualifier(sym, &xdt->right, "&&", FALSE);         fl |= 1; break;
+#endif
+        case 'I': append_extended_qualifier(sym, &xdt->right, "__restrict", TRUE);  fl |= 2; break;
+        default: if (fl == 1 || (fl == 3 && (sym->flags & UNDNAME_NO_MS_KEYWORDS))) xdt->flags = DT_NO_LRSEP_WS; return;
         }
         sym->current++;
     }
-    switch (*sym->current++)
+}
+
+/******************************************************************
+ *		get_qualifier
+ * Parses the type qualifier. Always returns static strings.
+ */
+static BOOL get_qualifier(struct parsed_symbol *sym, struct datatype_t *xdt, const char** pclass)
+{
+    char ch;
+    const char* qualif;
+
+    get_extended_qualifier(sym, xdt);
+    switch (ch = *sym->current++)
     {
-    case 'A': *ret = NULL; break;
-    case 'B': *ret = "const"; break;
-    case 'C': *ret = "volatile"; break;
-    case 'D': *ret = "const volatile"; break;
+    case 'A': qualif = NULL; break;
+    case 'B': qualif = "const"; break;
+    case 'C': qualif = "volatile"; break;
+    case 'D': qualif = "const volatile"; break;
+    case 'Q': qualif = NULL; break;
+    case 'R': qualif = "const"; break;
+    case 'S': qualif = "volatile"; break;
+    case 'T': qualif = "const volatile"; break;
     default: return FALSE;
     }
+    if (qualif)
+    {
+        xdt->flags &= ~DT_NO_LRSEP_WS;
+        xdt->left = xdt->left ? str_printf(sym, "%s %s", qualif, xdt->left) : qualif;
+    }
+    if (ch >= 'Q' && ch <= 'T') /* pointer to member, fetch class */
+    {
+        const char* class = get_class_name(sym);
+        if (!class) return FALSE;
+        if (!pclass)
+        {
+            FIXME("Got pointer to class %s member without storage\n", class);
+            return FALSE;
+        }
+        *pclass = class;
+    }
+    else if (pclass) *pclass = NULL;
     return TRUE;
 }
 
-static BOOL get_modified_type(struct datatype_t *ct, struct parsed_symbol* sym,
-                              struct array *pmt_ref, char modif, BOOL in_args)
+static BOOL get_function_qualifier(struct parsed_symbol *sym, const char** qualif)
 {
-    const char* modifier;
-    const char* str_modif;
-    const char *ptr_modif = "";
+    struct datatype_t   xdt;
 
-    if (*sym->current == 'E')
-    {
-        if (!(sym->flags & UNDNAME_NO_MS_KEYWORDS))
-        {
-            if (sym->flags & UNDNAME_NO_LEADING_UNDERSCORES)
-                ptr_modif = " ptr64";
-            else
-                ptr_modif = " __ptr64";
-        }
-        sym->current++;
-    }
+    if (!get_qualifier(sym, &xdt, NULL)) return FALSE;
+    *qualif = (xdt.left || xdt.right) ?
+        str_printf(sym, "%s%s%s", xdt.left, (xdt.flags & DT_NO_LRSEP_WS) ? "" : " ", xdt.right) : NULL;
+    return TRUE;
+}
 
-    switch (modif)
+static BOOL get_qualified_type(struct datatype_t *ct, struct parsed_symbol* sym,
+                              struct array *pmt_ref, char qualif, enum datatype_flags flags)
+{
+    struct datatype_t xdt1;
+    struct datatype_t xdt2;
+    const char* ref;
+    const char* str_qualif;
+    const char* class;
+
+    get_extended_qualifier(sym, &xdt1);
+
+    switch (qualif)
     {
-    case 'A': str_modif = str_printf(sym, " &%s", ptr_modif); break;
-    case 'B': str_modif = str_printf(sym, " &%s volatile", ptr_modif); break;
-    case 'P': str_modif = str_printf(sym, " *%s", ptr_modif); break;
-    case 'Q': str_modif = str_printf(sym, " *%s const", ptr_modif); break;
-    case 'R': str_modif = str_printf(sym, " *%s volatile", ptr_modif); break;
-    case 'S': str_modif = str_printf(sym, " *%s const volatile", ptr_modif); break;
-    case '?': str_modif = ""; break;
+    case 'A': ref = " &";  str_qualif = NULL;              break;
+    case 'B': ref = " &";  str_qualif = " volatile";       break;
+    case 'P': ref = " *";  str_qualif = NULL;              break;
+    case 'Q': ref = " *";  str_qualif = " const";          break;
+    case 'R': ref = " *";  str_qualif = " volatile";       break;
+    case 'S': ref = " *";  str_qualif = " const volatile"; break;
+    case '?': ref = NULL;  str_qualif = NULL;              break;
+    case '$': ref = " &&"; str_qualif = NULL;              break;
     default: return FALSE;
     }
+    ct->right = NULL;
+    ct->flags = 0;
 
-    if (get_modifier(sym, &modifier, &ptr_modif))
+    if (get_qualifier(sym, &xdt2, &class))
     {
         unsigned            mark = sym->stack.num;
         struct datatype_t   sub_ct;
 
+        if (ref || str_qualif || xdt1.left || xdt1.right)
+        {
+            if (class)
+                ct->left = str_printf(sym, "%s%s%s%s::%s%s%s",
+                                      xdt1.left ? " " : NULL, xdt1.left,
+                                      class ? " " : NULL, class, ref ? ref + 1 : NULL,
+                                      xdt1.right ? " " : NULL, xdt1.right, str_qualif);
+            else
+                ct->left = str_printf(sym, "%s%s%s%s%s%s",
+                                      xdt1.left ? " " : NULL, xdt1.left, ref,
+                                      xdt1.right ? " " : NULL, xdt1.right, str_qualif);
+        }
+        else
+            ct->left = NULL;
         /* multidimensional arrays */
         if (*sym->current == 'Y')
         {
@@ -468,36 +549,31 @@ static BOOL get_modified_type(struct datatype_t *ct, struct parsed_symbol* sym,
             if (!(n1 = get_number(sym))) return FALSE;
             num = atoi(n1);
 
-            if (str_modif[0] == ' ' && !modifier)
-                str_modif++;
-
-            if (modifier)
-            {
-                str_modif = str_printf(sym, " (%s%s)", modifier, str_modif);
-                modifier = NULL;
-            }
-            else
-                str_modif = str_printf(sym, " (%s)", str_modif);
+            ct->left = str_printf(sym, " (%s%s", xdt2.left, ct->left && !xdt2.left ? ct->left + 1 : ct->left);
+            ct->right = ")";
+            xdt2.left = NULL;
 
             while (num--)
-                str_modif = str_printf(sym, "%s[%s]", str_modif, get_number(sym));
+                ct->right = str_printf(sym, "%s[%s]", ct->right, get_number(sym));
         }
 
         /* Recurse to get the referred-to type */
-        if (!demangle_datatype(sym, &sub_ct, pmt_ref, FALSE))
+        if (!demangle_datatype(sym, &sub_ct, pmt_ref, 0))
             return FALSE;
-        if (modifier)
-            ct->left = str_printf(sym, "%s %s%s", sub_ct.left, modifier, str_modif );
-        else
-        {
-            /* don't insert a space between duplicate '*' */
-            if (!in_args && str_modif[0] && str_modif[1] == '*' && sub_ct.left[strlen(sub_ct.left)-1] == '*')
-                str_modif++;
-            ct->left = str_printf(sym, "%s%s", sub_ct.left, str_modif );
-        }
-        ct->right = sub_ct.right;
+        if (sub_ct.flags & DT_NO_LEADING_WS)
+            ct->left++;
+        ct->left = str_printf(sym, "%s%s%s%s%s", sub_ct.left, xdt2.left ? " " : NULL,
+                              xdt2.left, ct->left,
+                              ((xdt2.left || str_qualif) && (flags & WS_AFTER_QUAL_IF)) ? " " : NULL);
+        if (sub_ct.right) ct->right = str_printf(sym, "%s%s", ct->right, sub_ct.right);
         sym->stack.num = mark;
     }
+    else if (ref || str_qualif || xdt1.left || xdt1.right)
+        ct->left = str_printf(sym, "%s%s%s%s%s%s",
+                              xdt1.left ? " " : NULL, xdt1.left, ref,
+                              xdt1.right ? " " : NULL, xdt1.right, str_qualif);
+    else
+        ct->left = NULL;
     return TRUE;
 }
 
@@ -772,12 +848,41 @@ static const char* get_extended_type(char c)
     case 'L': type_string = "__int128"; break;
     case 'M': type_string = "unsigned __int128"; break;
     case 'N': type_string = "bool"; break;
+    case 'Q': type_string = "char8_t"; break;
+    case 'S': type_string = "char16_t"; break;
+    case 'U': type_string = "char32_t"; break;
     case 'W': type_string = "wchar_t"; break;
     default:  type_string = NULL; break;
     }
     return type_string;
 }
 
+struct function_signature
+{
+    const char*             call_conv;
+    const char*             exported;
+    struct datatype_t       return_ct;
+    const char*             arguments;
+};
+
+static BOOL get_function_signature(struct parsed_symbol* sym, struct array* pmt_ref,
+                                   struct function_signature* fs)
+{
+    unsigned mark = sym->stack.num;
+
+    if (!get_calling_convention(*sym->current++,
+                                &fs->call_conv, &fs->exported,
+                                sym->flags & ~UNDNAME_NO_ALLOCATION_LANGUAGE) ||
+        !demangle_datatype(sym, &fs->return_ct, pmt_ref, FALSE))
+        return FALSE;
+
+    if (!(fs->arguments = get_args(sym, pmt_ref, TRUE, '(', ')')))
+        return FALSE;
+    sym->stack.num = mark;
+
+    return TRUE;
+}
+
 /*******************************************************************
  *         demangle_datatype
  *
@@ -786,14 +891,15 @@ static const char* get_extended_type(char c)
  * char** = (pointer to (pointer to (char)))
  */
 static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
-                              struct array* pmt_ref, BOOL in_args)
+                              struct array* pmt_ref, enum datatype_flags flags)
 {
     char                dt;
     BOOL                add_pmt = TRUE;
 
     assert(ct);
     ct->left = ct->right = NULL;
-    
+    ct->flags = 0;
+
     switch (dt = *sym->current++)
     {
     case '_':
@@ -833,7 +939,7 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
         break;
     case '?':
         /* not all the time is seems */
-        if (in_args)
+        if (flags & IN_ARGS)
         {
             const char*   ptr;
             if (!(ptr = get_number(sym))) goto done;
@@ -841,17 +947,17 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
         }
         else
         {
-            if (!get_modified_type(ct, sym, pmt_ref, '?', in_args)) goto done;
+            if (!get_qualified_type(ct, sym, pmt_ref, '?', flags)) goto done;
         }
         break;
     case 'A': /* reference */
     case 'B': /* volatile reference */
-        if (!get_modified_type(ct, sym, pmt_ref, dt, in_args)) goto done;
+        if (!get_qualified_type(ct, sym, pmt_ref, dt, flags)) goto done;
         break;
     case 'Q': /* const pointer */
     case 'R': /* volatile pointer */
     case 'S': /* const volatile pointer */
-        if (!get_modified_type(ct, sym, pmt_ref, in_args ? dt : 'P', in_args)) goto done;
+        if (!get_qualified_type(ct, sym, pmt_ref, (flags & IN_ARGS) ? dt : 'P', flags)) goto done;
         break;
     case 'P': /* Pointer */
         if (isdigit(*sym->current))
@@ -862,67 +968,40 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
              *   others who knows.. */
             if (*sym->current == '8')
             {
-                char*                   args = NULL;
-                const char*             call_conv;
-                const char*             exported;
-                struct datatype_t       sub_ct;
-                unsigned                mark = sym->stack.num;
-                const char*             class;
-                const char*             modifier;
-                const char*             ptr_modif;
+                struct function_signature       fs;
+                const char*                     class;
+                const char*                     function_qualifier;
 
                 sym->current++;
 
                 if (!(class = get_class_name(sym)))
                     goto done;
-                if (!get_modifier(sym, &modifier, &ptr_modif))
+                if (!get_function_qualifier(sym, &function_qualifier))
                     goto done;
-                if (modifier)
-                    modifier = str_printf(sym, "%s %s", modifier, ptr_modif);
-                else if(ptr_modif)
-                    modifier = str_printf(sym, " %s", ptr_modif);
-                if (!get_calling_convention(*sym->current++,
-                            &call_conv, &exported,
-                            sym->flags & ~UNDNAME_NO_ALLOCATION_LANGUAGE))
-                    goto done;
-                if (!demangle_datatype(sym, &sub_ct, pmt_ref, FALSE))
-                    goto done;
-
-                args = get_args(sym, pmt_ref, TRUE, '(', ')');
-                if (!args) goto done;
-                sym->stack.num = mark;
+                if (!get_function_signature(sym, pmt_ref, &fs))
+                     goto done;
 
                 ct->left  = str_printf(sym, "%s%s (%s %s::*",
-                        sub_ct.left, sub_ct.right, call_conv, class);
-                ct->right = str_printf(sym, ")%s%s", args, modifier);
+                                       fs.return_ct.left, fs.return_ct.right, fs.call_conv, class);
+                ct->right = str_printf(sym, ")%s%s", fs.arguments, function_qualifier);
             }
             else if (*sym->current == '6')
             {
-                char*                   args = NULL;
-                const char*             call_conv;
-                const char*             exported;
-                struct datatype_t       sub_ct;
-                unsigned                mark = sym->stack.num;
+                struct function_signature       fs;
 
                 sym->current++;
 
-                if (!get_calling_convention(*sym->current++,
-                                            &call_conv, &exported, 
-                                            sym->flags & ~UNDNAME_NO_ALLOCATION_LANGUAGE) ||
-                    !demangle_datatype(sym, &sub_ct, pmt_ref, FALSE))
-                    goto done;
+                if (!get_function_signature(sym, pmt_ref, &fs))
+                     goto done;
 
-                args = get_args(sym, pmt_ref, TRUE, '(', ')');
-                if (!args) goto done;
-                sym->stack.num = mark;
-
-                ct->left  = str_printf(sym, "%s%s (%s*", 
-                                       sub_ct.left, sub_ct.right, call_conv);
-                ct->right = str_printf(sym, ")%s", args);
+                ct->left  = str_printf(sym, "%s%s (%s*",
+                                       fs.return_ct.left, fs.return_ct.right, fs.call_conv);
+                ct->flags = DT_NO_LEADING_WS;
+                ct->right = str_printf(sym, ")%s", fs.arguments);
             }
             else goto done;
 	}
-	else if (!get_modified_type(ct, sym, pmt_ref, 'P', in_args)) goto done;
+	else if (!get_qualified_type(ct, sym, pmt_ref, 'P', flags)) goto done;
         break;
     case 'W':
         if (*sym->current == '4')
@@ -989,7 +1068,22 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
             }
             break;
         case '$':
-            if (*sym->current == 'B')
+            if (*sym->current == 'A')
+            {
+                sym->current++;
+                if (*sym->current == '6')
+                {
+                    struct function_signature fs;
+
+                    sym->current++;
+
+                    if (!get_function_signature(sym, pmt_ref, &fs))
+                        goto done;
+                    ct->left = str_printf(sym, "%s%s %s%s",
+                                          fs.return_ct.left, fs.return_ct.right, fs.call_conv, fs.arguments);
+                }
+            }
+            else if (*sym->current == 'B')
             {
                 unsigned            mark = sym->stack.num;
                 struct datatype_t   sub_ct;
@@ -1010,7 +1104,7 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
                         arr = str_printf(sym, "%s[%s]", arr, get_number(sym));
                 }
 
-                if (!demangle_datatype(sym, &sub_ct, pmt_ref, FALSE)) goto done;
+                if (!demangle_datatype(sym, &sub_ct, pmt_ref, 0)) goto done;
 
                 if (arr)
                     ct->left = str_printf(sym, "%s %s", sub_ct.left, arr);
@@ -1021,12 +1115,17 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
             }
             else if (*sym->current == 'C')
             {
-                const char *ptr, *ptr_modif;
+                struct datatype_t xdt;
 
                 sym->current++;
-                if (!get_modifier(sym, &ptr, &ptr_modif)) goto done;
-                if (!demangle_datatype(sym, ct, pmt_ref, in_args)) goto done;
-                ct->left = str_printf(sym, "%s %s", ct->left, ptr);
+                if (!get_qualifier(sym, &xdt, NULL)) goto done;
+                if (!demangle_datatype(sym, ct, pmt_ref, flags)) goto done;
+                ct->left = str_printf(sym, "%s %s", ct->left, xdt.left);
+            }
+            else if (*sym->current == 'Q')
+            {
+                sym->current++;
+                if (!get_qualified_type(ct, sym, pmt_ref, '$', flags)) goto done;
             }
             break;
         }
@@ -1035,7 +1134,7 @@ static BOOL demangle_datatype(struct parsed_symbol* sym, struct datatype_t* ct,
         ERR("Unknown type %c\n", dt);
         break;
     }
-    if (add_pmt && pmt_ref && in_args)
+    if (add_pmt && pmt_ref && (flags & IN_ARGS))
     {
         /* left and right are pushed as two separate strings */
         if (!str_array_push(sym, ct->left ? ct->left : "", -1, pmt_ref) ||
@@ -1056,8 +1155,7 @@ static BOOL handle_data(struct parsed_symbol* sym)
 {
     const char*         access = NULL;
     const char*         member_type = NULL;
-    const char*         modifier = NULL;
-    const char*         ptr_modif;
+    struct datatype_t   xdt = {NULL};
     struct datatype_t   ct;
     char*               name = NULL;
     BOOL                ret = FALSE;
@@ -1098,20 +1196,21 @@ static BOOL handle_data(struct parsed_symbol* sym)
         {
             unsigned mark = sym->stack.num;
             struct array pmt;
+            const char* class;
 
             str_array_init(&pmt);
 
-            if (!demangle_datatype(sym, &ct, &pmt, FALSE)) goto done;
-            if (!get_modifier(sym, &modifier, &ptr_modif)) goto done;
-            if (modifier && ptr_modif) modifier = str_printf(sym, "%s %s", modifier, ptr_modif);
-            else if (!modifier) modifier = ptr_modif;
+            if (!demangle_datatype(sym, &ct, &pmt, 0)) goto done;
+            if (!get_qualifier(sym, &xdt, &class)) goto done; /* class doesn't seem to be displayed */
+            if (xdt.left && xdt.right) xdt.left = str_printf(sym, "%s %s", xdt.left, xdt.right);
+            else if (!xdt.left) xdt.left = xdt.right;
             sym->stack.num = mark;
         }
         break;
     case '6' : /* compiler generated static */
     case '7' : /* compiler generated static */
         ct.left = ct.right = NULL;
-        if (!get_modifier(sym, &modifier, &ptr_modif)) goto done;
+        if (!get_qualifier(sym, &xdt, NULL)) goto done;
         if (*sym->current != '@')
         {
             char*       cls = NULL;
@@ -1123,16 +1222,16 @@ static BOOL handle_data(struct parsed_symbol* sym)
         break;
     case '8':
     case '9':
-        modifier = ct.left = ct.right = NULL;
+        xdt.left = ct.left = ct.right = NULL;
         break;
     default: goto done;
     }
-    if (sym->flags & UNDNAME_NAME_ONLY) ct.left = ct.right = modifier = NULL;
+    if (sym->flags & UNDNAME_NAME_ONLY) ct.left = ct.right = xdt.left = NULL;
 
     sym->result = str_printf(sym, "%s%s%s%s%s%s%s%s", access,
-                             member_type, ct.left, 
-                             modifier && ct.left ? " " : NULL, modifier, 
-                             modifier || ct.left ? " " : NULL, name, ct.right);
+                             member_type, ct.left,
+                             xdt.left && ct.left ? " " : NULL, xdt.left,
+                             xdt.left || ct.left ? " " : NULL, name, ct.right);
     ret = TRUE;
 done:
     return ret;
@@ -1151,7 +1250,7 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
     const char*         member_type = NULL;
     struct datatype_t   ct_ret;
     const char*         call_conv;
-    const char*         modifier = NULL;
+    const char*         function_qualifier = NULL;
     const char*         exported;
     const char*         args_str = NULL;
     const char*         name = NULL;
@@ -1279,11 +1378,8 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
     if (has_args && (accmem == '$' ||
                 (accmem <= 'X' && (accmem - 'A') % 8 != 2 && (accmem - 'A') % 8 != 3)))
     {
-        const char *ptr_modif;
         /* Implicit 'this' pointer */
-        /* If there is an implicit this pointer, const modifier follows */
-        if (!get_modifier(sym, &modifier, &ptr_modif)) goto done;
-        if (modifier || ptr_modif) modifier = str_printf(sym, "%s %s", modifier, ptr_modif);
+        if (!get_function_qualifier(sym, &function_qualifier)) goto done;
     }
 
     if (!get_calling_convention(*sym->current++, &call_conv, &exported,
@@ -1301,21 +1397,21 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
     }
     else if (has_ret)
     {
-        if (!demangle_datatype(sym, &ct_ret, &array_pmt, FALSE))
+        if (!demangle_datatype(sym, &ct_ret, &array_pmt, cast_op ? WS_AFTER_QUAL_IF : 0))
             goto done;
     }
     if (!has_ret || sym->flags & UNDNAME_NO_FUNCTION_RETURNS)
         ct_ret.left = ct_ret.right = NULL;
     if (cast_op)
     {
-        name = str_printf(sym, "%s%s%s", name, ct_ret.left, ct_ret.right);
+        name = str_printf(sym, "%s %s%s", name, ct_ret.left, ct_ret.right);
         ct_ret.left = ct_ret.right = NULL;
     }
 
     mark = sym->stack.num;
     if (has_args && !(args_str = get_args(sym, &array_pmt, TRUE, '(', ')'))) goto done;
-    if (sym->flags & UNDNAME_NAME_ONLY) args_str = modifier = NULL;
-    if (sym->flags & UNDNAME_NO_THISTYPE) modifier = NULL;
+    if (sym->flags & UNDNAME_NAME_ONLY) args_str = function_qualifier = NULL;
+    if (sym->flags & UNDNAME_NO_THISTYPE) function_qualifier = NULL;
     sym->stack.num = mark;
 
     /* Note: '()' after 'Z' means 'throws', but we don't care here
@@ -1325,7 +1421,7 @@ static BOOL handle_method(struct parsed_symbol* sym, BOOL cast_op)
                              access, member_type, ct_ret.left,
                              (ct_ret.left && !ct_ret.right) ? " " : NULL,
                              call_conv, call_conv ? " " : NULL, exported,
-                             name, args_str, modifier, ct_ret.right);
+                             name, args_str, function_qualifier, ct_ret.right);
     ret = TRUE;
 done:
     return ret;
@@ -1338,15 +1434,19 @@ done:
 static BOOL symbol_demangle(struct parsed_symbol* sym)
 {
     BOOL                ret = FALSE;
-    unsigned            do_after = 0;
-    static CHAR         dashed_null[] = "--null--";
+    enum {
+        PP_NONE,
+        PP_CONSTRUCTOR,
+        PP_DESTRUCTOR,
+        PP_CAST_OPERATOR,
+    } post_process = PP_NONE;
 
     /* FIXME seems wrong as name, as it demangles a simple data type */
     if (sym->flags & UNDNAME_NO_ARGUMENTS)
     {
         struct datatype_t   ct;
 
-        if (demangle_datatype(sym, &ct, NULL, FALSE))
+        if (demangle_datatype(sym, &ct, NULL, 0))
         {
             sym->result = str_printf(sym, "%s%s", ct.left, ct.right);
             ret = TRUE;
@@ -1359,21 +1459,22 @@ static BOOL symbol_demangle(struct parsed_symbol* sym)
     sym->current++;
 
     /* Then function name or operator code */
-    if (*sym->current == '?' && (sym->current[1] != '$' || sym->current[2] == '?'))
+    if (*sym->current == '?')
     {
         const char* function_name = NULL;
+        BOOL in_template = FALSE;
 
-        if (sym->current[1] == '$')
+        if (sym->current[1] == '$' && sym->current[2] == '?')
         {
-            do_after = 6;
+            in_template = TRUE;
             sym->current += 2;
         }
 
         /* C++ operator code (one character, or two if the first is '_') */
         switch (*++sym->current)
         {
-        case '0': do_after = 1; break;
-        case '1': do_after = 2; break;
+        case '0': function_name = ""; post_process = PP_CONSTRUCTOR; break;
+        case '1': function_name = ""; post_process = PP_DESTRUCTOR; break;
         case '2': function_name = "operator new"; break;
         case '3': function_name = "operator delete"; break;
         case '4': function_name = "operator="; break;
@@ -1383,7 +1484,7 @@ static BOOL symbol_demangle(struct parsed_symbol* sym)
         case '8': function_name = "operator=="; break;
         case '9': function_name = "operator!="; break;
         case 'A': function_name = "operator[]"; break;
-        case 'B': function_name = "operator "; do_after = 3; break;
+        case 'B': function_name = "operator"; post_process = PP_CAST_OPERATOR; break;
         case 'C': function_name = "operator->"; break;
         case 'D': function_name = "operator*"; break;
         case 'E': function_name = "operator++"; break;
@@ -1423,7 +1524,10 @@ static BOOL symbol_demangle(struct parsed_symbol* sym)
             case '9': function_name = "`vcall'"; break;
             case 'A': function_name = "`typeof'"; break;
             case 'B': function_name = "`local static guard'"; break;
-            case 'C': function_name = "`string'"; do_after = 4; break;
+            case 'C': sym->result = (char*)"`string'"; /* string literal: followed by string encoding (native never undecode it) */
+                /* FIXME: should unmangle the whole string for error reporting */
+                if (*sym->current && sym->current[strlen(sym->current) - 1] == '@') ret = TRUE;
+                goto done;
             case 'D': function_name = "`vbase destructor'"; break;
             case 'E': function_name = "`vector deleting destructor'"; break;
             case 'F': function_name = "`default constructor closure'"; break;
@@ -1443,12 +1547,9 @@ static BOOL symbol_demangle(struct parsed_symbol* sym)
                 case '0':
                     {
                         struct datatype_t       ct;
-                        struct array pmt;
 
                         sym->current++;
-                        str_array_init(&pmt);
-                        demangle_datatype(sym, &ct, &pmt, FALSE);
-                        if (!demangle_datatype(sym, &ct, NULL, FALSE))
+                        if (!demangle_datatype(sym, &ct, NULL, 0))
                             goto done;
                         function_name = str_printf(sym, "%s%s `RTTI Type Descriptor'",
                                                    ct.left, ct.right);
@@ -1482,43 +1583,47 @@ static BOOL symbol_demangle(struct parsed_symbol* sym)
             case 'V': function_name = "operator delete[]"; break;
             case 'X': function_name = "`placement delete closure'"; break;
             case 'Y': function_name = "`placement delete[] closure'"; break;
+            case '_':
+                switch (*++sym->current)
+                {
+                case 'K':
+                    sym->current++;
+                    function_name = str_printf(sym, "operator \"\" %s", get_literal_string(sym));
+                    --sym->current;
+                    break;
+                default:
+                    FIXME("Unknown operator: __%c\n", *sym->current);
+                    return FALSE;
+                }
+                break;
             default:
                 ERR("Unknown operator: _%c\n", *sym->current);
                 return FALSE;
             }
             break;
+        case '$':
+            sym->current++;
+            if (!(function_name = get_template_name(sym))) goto done;
+            --sym->current;
+            break;
         default:
             /* FIXME: Other operators */
             ERR("Unknown operator: %c\n", *sym->current);
             return FALSE;
         }
         sym->current++;
-        switch (do_after)
+        if (in_template)
         {
-        case 1: case 2:
-            if (!str_array_push(sym, dashed_null, -1, &sym->stack))
-                return FALSE;
-            break;
-        case 4:
-            sym->result = (char*)function_name;
-            ret = TRUE;
-            goto done;
-        case 6:
-            {
-                char *args;
-                struct array array_pmt;
+            const char *args;
+            struct array array_pmt;
 
-                str_array_init(&array_pmt);
-                args = get_args(sym, &array_pmt, FALSE, '<', '>');
-                if (args != NULL) function_name = str_printf(sym, "%s%s", function_name, args);
-                sym->names.num = 0;
-            }
-            /* fall through */
-        default:
-            if (!str_array_push(sym, function_name, -1, &sym->stack))
-                return FALSE;
-            break;
+            str_array_init(&array_pmt);
+            args = get_args(sym, &array_pmt, FALSE, '<', '>');
+            if (args) function_name = function_name ? str_printf(sym, "%s%s", function_name, args) : args;
+            sym->names.num = 0;
         }
+        if (!str_array_push(sym, function_name, -1, &sym->stack))
+            return FALSE;
     }
     else if (*sym->current == '$')
     {
@@ -1528,8 +1633,6 @@ static BOOL symbol_demangle(struct parsed_symbol* sym)
         ret = (sym->result = get_template_name(sym)) != NULL;
         goto done;
     }
-    else if (*sym->current == '?' && sym->current[1] == '$')
-        do_after = 5;
 
     /* Either a class name, or '@' if the symbol is not a class member */
     switch (*sym->current)
@@ -1542,32 +1645,27 @@ static BOOL symbol_demangle(struct parsed_symbol* sym)
         break;
     }
 
-    switch (do_after)
+    switch (post_process)
     {
-    case 0: default: break;
-    case 1: case 2:
+    case PP_NONE: default: break;
+    case PP_CONSTRUCTOR: case PP_DESTRUCTOR:
         /* it's time to set the member name for ctor & dtor */
         if (sym->stack.num <= 1) goto done;
-        if (do_after == 1)
-            sym->stack.elts[0] = sym->stack.elts[1];
-        else
-            sym->stack.elts[0] = str_printf(sym, "~%s", sym->stack.elts[1]);
+        sym->stack.elts[0] = str_printf(sym, "%s%s%s", post_process == PP_DESTRUCTOR ? "~" : NULL,
+                                        sym->stack.elts[1], sym->stack.elts[0]);
         /* ctors and dtors don't have return type */
         sym->flags |= UNDNAME_NO_FUNCTION_RETURNS;
         break;
-    case 3:
+    case PP_CAST_OPERATOR:
         sym->flags &= ~UNDNAME_NO_FUNCTION_RETURNS;
         break;
-    case 5:
-        sym->names.start++;
-        break;
     }
 
     /* Function/Data type and access level */
     if (*sym->current >= '0' && *sym->current <= '9')
         ret = handle_data(sym);
     else if ((*sym->current >= 'A' && *sym->current <= 'Z') || *sym->current == '$')
-        ret = handle_method(sym, do_after == 3);
+        ret = handle_method(sym, post_process == PP_CAST_OPERATOR);
     else ret = FALSE;
 done:
     if (ret) assert(sym->result);
