@@ -274,7 +274,7 @@ static ULONG WINAPI BindStatusCallback_AddRef(IBindStatusCallback *iface)
     BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref = %d\n", This, ref);
+    TRACE("%p, refcount %ld.\n", iface, ref);
 
     return ref;
 }
@@ -284,7 +284,7 @@ static ULONG WINAPI BindStatusCallback_Release(IBindStatusCallback *iface)
     BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref = %d\n", This, ref);
+    TRACE("%p, refcount %ld.\n", iface, ref);
 
     if (!ref)
     {
@@ -302,7 +302,7 @@ static HRESULT WINAPI BindStatusCallback_OnStartBinding(IBindStatusCallback *ifa
 {
     BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
-    TRACE("(%p)->(%d %p)\n", This, reserved, pbind);
+    TRACE("%p, %ld, %p.\n", iface, reserved, pbind);
 
     if (!pbind) return E_INVALIDARG;
 
@@ -325,9 +325,7 @@ static HRESULT WINAPI BindStatusCallback_GetPriority(IBindStatusCallback *iface,
 
 static HRESULT WINAPI BindStatusCallback_OnLowResource(IBindStatusCallback *iface, DWORD reserved)
 {
-    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
-
-    TRACE("(%p)->(%d)\n", This, reserved);
+    TRACE("%p, %ld.\n", iface, reserved);
 
     return E_NOTIMPL;
 }
@@ -335,9 +333,7 @@ static HRESULT WINAPI BindStatusCallback_OnLowResource(IBindStatusCallback *ifac
 static HRESULT WINAPI BindStatusCallback_OnProgress(IBindStatusCallback *iface, ULONG ulProgress,
         ULONG ulProgressMax, ULONG ulStatusCode, LPCWSTR szStatusText)
 {
-    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
-
-    TRACE("(%p)->(%u %u %u %s)\n", This, ulProgress, ulProgressMax, ulStatusCode,
+    TRACE("%p, %lu, %lu, %lu, %s.\n", iface, ulProgress, ulProgressMax, ulStatusCode,
             debugstr_w(szStatusText));
 
     return S_OK;
@@ -348,7 +344,7 @@ static HRESULT WINAPI BindStatusCallback_OnStopBinding(IBindStatusCallback *ifac
 {
     BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
-    TRACE("(%p)->(0x%08x %s)\n", This, hr, debugstr_w(error));
+    TRACE("%p, %#lx, %s.\n", iface, hr, debugstr_w(error));
 
     if (This->binding)
     {
@@ -403,7 +399,7 @@ static HRESULT WINAPI BindStatusCallback_OnDataAvailable(IBindStatusCallback *if
     BYTE buf[4096];
     HRESULT hr;
 
-    TRACE("(%p)->(%08x %d %p %p)\n", This, flags, size, format, stgmed);
+    TRACE("%p, %#lx, %lu, %p, %p.\n", iface, flags, size, format, stgmed);
 
     do
     {
@@ -474,7 +470,7 @@ static HRESULT WINAPI BSCHttpNegotiate_BeginningTransaction(IHttpNegotiate *ifac
     WCHAR *buff, *ptr;
     int size = 0;
 
-    TRACE("(%p)->(%s %s %d %p)\n", This, debugstr_w(url), debugstr_w(headers), reserved, add_headers);
+    TRACE("%p, %s, %s, %ld, %p.\n", iface, debugstr_w(url), debugstr_w(headers), reserved, add_headers);
 
     *add_headers = NULL;
 
@@ -576,7 +572,7 @@ static HRESULT WINAPI BSCHttpNegotiate_OnResponse(IHttpNegotiate *iface, DWORD c
 {
     BindStatusCallback *This = impl_from_IHttpNegotiate(iface);
 
-    TRACE("(%p)->(%d %s %s %p)\n", This, code, debugstr_w(resp_headers),
+    TRACE("%p, %ld, %s, %s, %p.\n", iface, code, debugstr_w(resp_headers),
           debugstr_w(req_headers), add_reqheaders);
 
     This->request->status = code;
@@ -687,7 +683,7 @@ static HRESULT BindStatusCallback_create(httprequest* This, BindStatusCallback *
     BindStatusCallback *bsc;
     IBindCtx *pbc;
     HRESULT hr;
-    int size;
+    LONG size;
 
     hr = CreateBindCtx(0, &pbc);
     if (hr != S_OK) return hr;
@@ -931,7 +927,7 @@ static HRESULT httprequest_open(httprequest *This, BSTR method, BSTR url,
     else
         hr = CreateUri(url, 0, 0, &uri);
     if(FAILED(hr)) {
-        WARN("Could not create IUri object: %08x\n", hr);
+        WARN("Could not create IUri object, hr %#lx.\n", hr);
         return hr;
     }
 
@@ -970,11 +966,11 @@ static HRESULT httprequest_open(httprequest *This, BSTR method, BSTR url,
                 uri = full_uri;
             }
             else
-                WARN("failed to create modified uri, 0x%08x\n", hr);
+                WARN("failed to create modified uri, hr %#lx.\n", hr);
             IUriBuilder_Release(builder);
         }
         else
-            WARN("IUriBuilder creation failed, 0x%08x\n", hr);
+            WARN("IUriBuilder creation failed, hr %#lx.\n", hr);
     }
 
     This->uri = uri;
@@ -1396,23 +1392,23 @@ static HRESULT WINAPI XMLHTTPRequest_QueryInterface(IXMLHTTPRequest *iface, REFI
 
 static ULONG WINAPI XMLHTTPRequest_AddRef(IXMLHTTPRequest *iface)
 {
-    httprequest *This = impl_from_IXMLHTTPRequest( iface );
-    ULONG ref = InterlockedIncrement( &This->ref );
-    TRACE("(%p)->(%u)\n", This, ref );
+    httprequest *request = impl_from_IXMLHTTPRequest(iface);
+    ULONG ref = InterlockedIncrement(&request->ref);
+    TRACE("%p, refcount %lu.\n", iface, ref);
     return ref;
 }
 
 static ULONG WINAPI XMLHTTPRequest_Release(IXMLHTTPRequest *iface)
 {
-    httprequest *This = impl_from_IXMLHTTPRequest( iface );
-    ULONG ref = InterlockedDecrement( &This->ref );
+    httprequest *request = impl_from_IXMLHTTPRequest(iface);
+    ULONG ref = InterlockedDecrement(&request->ref);
 
-    TRACE("(%p)->(%u)\n", This, ref );
+    TRACE("%p, refcount %lu.\n", iface, ref);
 
-    if ( ref == 0 )
+    if (!ref)
     {
-        httprequest_release( This );
-        heap_free( This );
+        httprequest_release(request);
+        heap_free(request);
     }
 
     return ref;
@@ -1420,9 +1416,7 @@ static ULONG WINAPI XMLHTTPRequest_Release(IXMLHTTPRequest *iface)
 
 static HRESULT WINAPI XMLHTTPRequest_GetTypeInfoCount(IXMLHTTPRequest *iface, UINT *pctinfo)
 {
-    httprequest *This = impl_from_IXMLHTTPRequest( iface );
-
-    TRACE("(%p)->(%p)\n", This, pctinfo);
+    TRACE("%p, %p.\n", iface, pctinfo);
 
     *pctinfo = 1;
 
@@ -1432,9 +1426,7 @@ static HRESULT WINAPI XMLHTTPRequest_GetTypeInfoCount(IXMLHTTPRequest *iface, UI
 static HRESULT WINAPI XMLHTTPRequest_GetTypeInfo(IXMLHTTPRequest *iface, UINT iTInfo,
         LCID lcid, ITypeInfo **ppTInfo)
 {
-    httprequest *This = impl_from_IXMLHTTPRequest( iface );
-
-    TRACE("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
+    TRACE("%p, %u, %lx,%p.\n", iface, iTInfo, lcid, ppTInfo);
 
     return get_typeinfo(IXMLHTTPRequest_tid, ppTInfo);
 }
@@ -1442,11 +1434,10 @@ static HRESULT WINAPI XMLHTTPRequest_GetTypeInfo(IXMLHTTPRequest *iface, UINT iT
 static HRESULT WINAPI XMLHTTPRequest_GetIDsOfNames(IXMLHTTPRequest *iface, REFIID riid,
         LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
 {
-    httprequest *This = impl_from_IXMLHTTPRequest( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%s %p %u %u %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
+    TRACE("%p, %s, %p, %u, %lx, %p.\n", iface, debugstr_guid(riid), rgszNames, cNames,
           lcid, rgDispId);
 
     if(!rgszNames || cNames == 0 || !rgDispId)
@@ -1466,18 +1457,16 @@ static HRESULT WINAPI XMLHTTPRequest_Invoke(IXMLHTTPRequest *iface, DISPID dispI
         LCID lcid, WORD wFlags, DISPPARAMS *pDispParams, VARIANT *pVarResult,
         EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
-    httprequest *This = impl_from_IXMLHTTPRequest( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
+    TRACE("%p, %ld, %s, %lx, %d, %p, %p, %p, %p.\n", iface, dispIdMember, debugstr_guid(riid),
           lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 
     hr = get_typeinfo(IXMLHTTPRequest_tid, &typeinfo);
     if(SUCCEEDED(hr))
     {
-        hr = ITypeInfo_Invoke(typeinfo, &This->IXMLHTTPRequest_iface, dispIdMember, wFlags,
-                pDispParams, pVarResult, pExcepInfo, puArgErr);
+        hr = ITypeInfo_Invoke(typeinfo, iface, dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
         ITypeInfo_Release(typeinfo);
     }
 
@@ -1743,13 +1732,14 @@ static HRESULT WINAPI httprequest_Safety_GetInterfaceSafetyOptions(IObjectSafety
 static HRESULT WINAPI httprequest_Safety_SetInterfaceSafetyOptions(IObjectSafety *iface, REFIID riid,
         DWORD mask, DWORD enabled)
 {
-    httprequest *This = impl_from_IObjectSafety(iface);
-    TRACE("(%p)->(%s %x %x)\n", This, debugstr_guid(riid), mask, enabled);
+    httprequest *request = impl_from_IObjectSafety(iface);
+
+    TRACE("%p, %s, %lx, %lx.\n", iface, debugstr_guid(riid), mask, enabled);
 
     if ((mask & ~safety_supported_options))
         return E_FAIL;
 
-    This->safeopt = (This->safeopt & ~mask) | (mask & enabled);
+    request->safeopt = (request->safeopt & ~mask) | (mask & enabled);
 
     return S_OK;
 }
@@ -1829,23 +1819,23 @@ static HRESULT WINAPI ServerXMLHTTPRequest_QueryInterface(IServerXMLHTTPRequest
 
 static ULONG WINAPI ServerXMLHTTPRequest_AddRef(IServerXMLHTTPRequest *iface)
 {
-    serverhttp *This = impl_from_IServerXMLHTTPRequest( iface );
-    ULONG ref = InterlockedIncrement( &This->req.ref );
-    TRACE("(%p)->(%u)\n", This, ref );
+    serverhttp *request = impl_from_IServerXMLHTTPRequest(iface);
+    ULONG ref = InterlockedIncrement(&request->req.ref);
+    TRACE("%p, refcount %lu.\n", iface, ref );
     return ref;
 }
 
 static ULONG WINAPI ServerXMLHTTPRequest_Release(IServerXMLHTTPRequest *iface)
 {
-    serverhttp *This = impl_from_IServerXMLHTTPRequest( iface );
-    ULONG ref = InterlockedDecrement( &This->req.ref );
+    serverhttp *request = impl_from_IServerXMLHTTPRequest(iface);
+    ULONG ref = InterlockedDecrement(&request->req.ref);
 
-    TRACE("(%p)->(%u)\n", This, ref );
+    TRACE("%p, refcount %lu.\n", iface, ref );
 
-    if ( ref == 0 )
+    if (!ref)
     {
-        httprequest_release( &This->req );
-        heap_free( This );
+        httprequest_release(&request->req);
+        heap_free(request);
     }
 
     return ref;
@@ -1864,9 +1854,7 @@ static HRESULT WINAPI ServerXMLHTTPRequest_GetTypeInfoCount(IServerXMLHTTPReques
 static HRESULT WINAPI ServerXMLHTTPRequest_GetTypeInfo(IServerXMLHTTPRequest *iface, UINT iTInfo,
         LCID lcid, ITypeInfo **ppTInfo)
 {
-    serverhttp *This = impl_from_IServerXMLHTTPRequest( iface );
-
-    TRACE("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
+    TRACE("%p, %u, %lx, %p.\n", iface, iTInfo, lcid, ppTInfo);
 
     return get_typeinfo(IServerXMLHTTPRequest_tid, ppTInfo);
 }
@@ -1874,11 +1862,10 @@ static HRESULT WINAPI ServerXMLHTTPRequest_GetTypeInfo(IServerXMLHTTPRequest *if
 static HRESULT WINAPI ServerXMLHTTPRequest_GetIDsOfNames(IServerXMLHTTPRequest *iface, REFIID riid,
         LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
 {
-    serverhttp *This = impl_from_IServerXMLHTTPRequest( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%s %p %u %u %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
+    TRACE("%p, %s, %p, %u, %lx, %p.\n", iface, debugstr_guid(riid), rgszNames, cNames,
           lcid, rgDispId);
 
     if(!rgszNames || cNames == 0 || !rgDispId)
@@ -1898,18 +1885,16 @@ static HRESULT WINAPI ServerXMLHTTPRequest_Invoke(IServerXMLHTTPRequest *iface,
         LCID lcid, WORD wFlags, DISPPARAMS *pDispParams, VARIANT *pVarResult,
         EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
-    serverhttp *This = impl_from_IServerXMLHTTPRequest( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
+    TRACE("%p, %ld, %s %lx, %d, %p, %p, %p, %p.\n", iface, dispIdMember, debugstr_guid(riid),
           lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 
     hr = get_typeinfo(IServerXMLHTTPRequest_tid, &typeinfo);
     if(SUCCEEDED(hr))
     {
-        hr = ITypeInfo_Invoke(typeinfo, &This->IServerXMLHTTPRequest_iface, dispIdMember, wFlags,
-                pDispParams, pVarResult, pExcepInfo, puArgErr);
+        hr = ITypeInfo_Invoke(typeinfo, iface, dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
         ITypeInfo_Release(typeinfo);
     }
 
@@ -2019,8 +2004,7 @@ static HRESULT WINAPI ServerXMLHTTPRequest_put_onreadystatechange(IServerXMLHTTP
 static HRESULT WINAPI ServerXMLHTTPRequest_setTimeouts(IServerXMLHTTPRequest *iface, LONG resolveTimeout, LONG connectTimeout,
     LONG sendTimeout, LONG receiveTimeout)
 {
-    serverhttp *This = impl_from_IServerXMLHTTPRequest( iface );
-    FIXME("(%p)->(%d %d %d %d): stub\n", This, resolveTimeout, connectTimeout, sendTimeout, receiveTimeout);
+    FIXME("%p, %ld, %ld, %ld, %ld: stub\n", iface, resolveTimeout, connectTimeout, sendTimeout, receiveTimeout);
     return S_OK;
 }
 
