@@ -26,6 +26,7 @@
 
 #include <assert.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <limits.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -58,6 +59,10 @@
 #include <libprocstat.h>
 #endif
 
+#ifdef __APPLE__
+#include <mach/mach.h>
+#endif
+
 #define NONAMELESSUNION
 #define NONAMELESSSTRUCT
 #include "ntstatus.h"
@@ -70,12 +75,11 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(thread);
 WINE_DECLARE_DEBUG_CHANNEL(seh);
+WINE_DECLARE_DEBUG_CHANNEL(threadname);
 
-#ifndef PTHREAD_STACK_MIN
-#define PTHREAD_STACK_MIN 16384
-#endif
+pthread_key_t teb_key = 0;
 
-static int nb_threads = 1;
+static LONG nb_threads = 1;
 
 static inline int get_unix_exit_code( NTSTATUS status )
 {
@@ -1068,7 +1072,7 @@ static void start_thread( TEB *teb )
     BOOL suspend;
 
     thread_data->pthread_id = pthread_self();
-    signal_init_thread( teb );
+    pthread_setspecific( teb_key, teb );
     server_init_thread( thread_data->start, &suspend );
     signal_start_thread( thread_data->start, thread_data->param, suspend, teb );
 }
@@ -1110,12 +1114,12 @@ void *get_cpu_area( USHORT machine )
     switch (cpu->Machine)
     {
     case IMAGE_FILE_MACHINE_I386: align = TYPE_ALIGNMENT(I386_CONTEXT); break;
-    case IMAGE_FILE_MACHINE_AMD64: align = TYPE_ALIGNMENT(ARM_CONTEXT); break;
-    case IMAGE_FILE_MACHINE_ARMNT: align = TYPE_ALIGNMENT(AMD64_CONTEXT); break;
+    case IMAGE_FILE_MACHINE_AMD64: align = TYPE_ALIGNMENT(AMD64_CONTEXT); break;
+    case IMAGE_FILE_MACHINE_ARMNT: align = TYPE_ALIGNMENT(ARM_CONTEXT); break;
     case IMAGE_FILE_MACHINE_ARM64: align = TYPE_ALIGNMENT(ARM64_NT_CONTEXT); break;
     default: return NULL;
     }
-    return (void *)(((ULONG_PTR)(cpu + 1) + align - 1) & ~(align - 1));
+    return (void *)(((ULONG_PTR)(cpu + 1) + align - 1) & ~((ULONG_PTR)align - 1));
 }
 
 
@@ -1156,7 +1160,8 @@ NTSTATUS init_thread_stack( TEB *teb, ULONG_PTR zero_bits, SIZE_T reserve_size,
 
 #ifdef _WIN64
         /* 32-bit stack */
-        if ((status = virtual_alloc_thread_stack( &stack, zero_bits, reserve_size, commit_size, 0 )))
+        if ((status = virtual_alloc_thread_stack( &stack, zero_bits ? zero_bits : 0x7fffffff,
+                                                  reserve_size, commit_size, 0 )))
             return status;
         wow_teb->Tib.StackBase = PtrToUlong( stack.StackBase );
         wow_teb->Tib.StackLimit = PtrToUlong( stack.StackLimit );
@@ -1229,7 +1234,8 @@ NTSTATUS WINAPI NtCreateThread( HANDLE *handle, ACCESS_MASK access, OBJECT_ATTRI
                                 HANDLE process, CLIENT_ID *id, CONTEXT *ctx, INITIAL_TEB *teb,
                                 BOOLEAN suspended )
 {
-    FIXME( "%p %d %p %p %p %p %p %d, stub!\n", handle, access, attr, process, id, ctx, teb, suspended );
+    FIXME( "%p %d %p %p %p %p %p %d, stub!\n",
+           handle, (int)access, attr, process, id, ctx, teb, suspended );
     return STATUS_NOT_IMPLEMENTED;
 }
 
@@ -1251,10 +1257,10 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
     DWORD tid = 0;
     int request_pipe[2];
     TEB *teb;
-    NTSTATUS status;
+    unsigned int status;
 
     if (flags & ~supported_flags)
-        FIXME( "Unsupported flags %#x.\n", flags );
+        FIXME( "Unsupported flags %#x.\n", (int)flags );
 
     if (zero_bits > 21 && zero_bits < 32) return STATUS_INVALID_PARAMETER_3;
 #ifndef _WIN64
@@ -1447,7 +1453,7 @@ void wait_suspend( CONTEXT *context )
  */
 NTSTATUS send_debug_event( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_chance )
 {
-    NTSTATUS ret;
+    unsigned int ret;
     DWORD i;
     obj_handle_t handle = 0;
     client_ptr_t params[EXCEPTION_MAXIMUM_PARAMETERS];
@@ -1491,7 +1497,7 @@ NTSTATUS send_debug_event( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_c
             ret = wine_server_call( req );
         }
         SERVER_END_REQ;
-        if (ret >= 0) contexts_from_server( context, server_contexts );
+        if (NT_SUCCESS(ret)) contexts_from_server( context, server_contexts );
     }
 
     pthread_sigmask( SIG_SETMASK, &old_set, NULL );
@@ -1517,20 +1523,29 @@ NTSTATUS WINAPI NtRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL
         ERR_(seh)("Process attempted to continue execution after noncontinuable exception.\n");
     else
         ERR_(seh)("Unhandled exception code %x flags %x addr %p\n",
-                  rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress );
+                  (int)rec->ExceptionCode, (int)rec->ExceptionFlags, rec->ExceptionAddress );
 
     NtTerminateProcess( NtCurrentProcess(), rec->ExceptionCode );
     return STATUS_SUCCESS;
 }
 
 
+/**********************************************************************
+ *           NtCurrentTeb   (NTDLL.@)
+ */
+TEB * WINAPI NtCurrentTeb(void)
+{
+    return pthread_getspecific( teb_key );
+}
+
+
 /***********************************************************************
  *              NtOpenThread   (NTDLL.@)
  */
 NTSTATUS WINAPI NtOpenThread( HANDLE *handle, ACCESS_MASK access,
                               const OBJECT_ATTRIBUTES *attr, const CLIENT_ID *id )
 {
-    NTSTATUS ret;
+    unsigned int ret;
 
     *handle = 0;
 
@@ -1552,7 +1567,7 @@ NTSTATUS WINAPI NtOpenThread( HANDLE *handle, ACCESS_MASK access,
  */
 NTSTATUS WINAPI NtSuspendThread( HANDLE handle, ULONG *count )
 {
-    NTSTATUS ret;
+    unsigned int ret;
 
     SERVER_START_REQ( suspend_thread )
     {
@@ -1572,7 +1587,7 @@ NTSTATUS WINAPI NtSuspendThread( HANDLE handle, ULONG *count )
  */
 NTSTATUS WINAPI NtResumeThread( HANDLE handle, ULONG *count )
 {
-    NTSTATUS ret;
+    unsigned int ret;
 
     SERVER_START_REQ( resume_thread )
     {
@@ -1612,21 +1627,23 @@ NTSTATUS WINAPI NtAlertThread( HANDLE handle )
  */
 NTSTATUS WINAPI NtTerminateThread( HANDLE handle, LONG exit_code )
 {
-    NTSTATUS ret;
-    BOOL self = (handle == GetCurrentThread());
+    unsigned int ret;
+    BOOL self;
 
-    if (!self || exit_code)
+    SERVER_START_REQ( terminate_thread )
     {
-        SERVER_START_REQ( terminate_thread )
-        {
-            req->handle    = wine_server_obj_handle( handle );
-            req->exit_code = exit_code;
-            ret = wine_server_call( req );
-            self = !ret && reply->self;
-        }
-        SERVER_END_REQ;
+        req->handle    = wine_server_obj_handle( handle );
+        req->exit_code = exit_code;
+        ret = wine_server_call( req );
+        self = !ret && reply->self;
+    }
+    SERVER_END_REQ;
+
+    if (self)
+    {
+        server_select( NULL, 0, SELECT_INTERRUPTIBLE, 0, NULL, NULL );
+        exit_thread( exit_code );
     }
-    if (self) exit_thread( exit_code );
     return ret;
 }
 
@@ -1637,7 +1654,7 @@ NTSTATUS WINAPI NtTerminateThread( HANDLE handle, LONG exit_code )
 NTSTATUS WINAPI NtQueueApcThread( HANDLE handle, PNTAPCFUNC func, ULONG_PTR arg1,
                                   ULONG_PTR arg2, ULONG_PTR arg3 )
 {
-    NTSTATUS ret;
+    unsigned int ret;
 
     SERVER_START_REQ( queue_apc )
     {
@@ -1665,7 +1682,7 @@ NTSTATUS set_thread_context( HANDLE handle, const void *context, BOOL *self, USH
 {
     context_t server_contexts[2];
     unsigned int count = 0;
-    NTSTATUS ret;
+    unsigned int ret;
 
     context_to_server( &server_contexts[count++], native_machine, context, machine );
     if (machine != native_machine)
@@ -1689,7 +1706,7 @@ NTSTATUS set_thread_context( HANDLE handle, const void *context, BOOL *self, USH
  */
 NTSTATUS get_thread_context( HANDLE handle, void *context, BOOL *self, USHORT machine )
 {
-    NTSTATUS ret;
+    unsigned int ret;
     HANDLE context_handle;
     context_t server_contexts[2];
     unsigned int count;
@@ -1828,6 +1845,82 @@ BOOL get_thread_times(int unix_pid, int unix_tid, LARGE_INTEGER *kernel_time, LA
 #endif
 }
 
+static void set_native_thread_name( HANDLE handle, const UNICODE_STRING *name )
+{
+#ifdef linux
+    unsigned int status;
+    char path[64], nameA[64];
+    int unix_pid, unix_tid, len, fd;
+
+    SERVER_START_REQ( get_thread_times )
+    {
+        req->handle = wine_server_obj_handle( handle );
+        status = wine_server_call( req );
+        if (status == STATUS_SUCCESS)
+        {
+            unix_pid = reply->unix_pid;
+            unix_tid = reply->unix_tid;
+        }
+    }
+    SERVER_END_REQ;
+
+    if (status != STATUS_SUCCESS || unix_pid == -1 || unix_tid == -1)
+        return;
+
+    if (unix_pid != getpid())
+    {
+        static int once;
+        if (!once++) FIXME("cross-process native thread naming not supported\n");
+        return;
+    }
+
+    len = ntdll_wcstoumbs( name->Buffer, name->Length / sizeof(WCHAR), nameA, sizeof(nameA), FALSE );
+    sprintf(path, "/proc/%u/task/%u/comm", unix_pid, unix_tid);
+    if ((fd = open( path, O_WRONLY )) != -1)
+    {
+        write( fd, nameA, len );
+        close( fd );
+    }
+#elif defined(__APPLE__)
+    /* pthread_setname_np() silently fails if the name is longer than 63 characters + null terminator */
+    char nameA[64];
+    NTSTATUS status;
+    int unix_pid, unix_tid, len, current_tid;
+
+    SERVER_START_REQ( get_thread_times )
+    {
+        req->handle = wine_server_obj_handle( handle );
+        status = wine_server_call( req );
+        if (status == STATUS_SUCCESS)
+        {
+            unix_pid = reply->unix_pid;
+            unix_tid = reply->unix_tid;
+        }
+    }
+    SERVER_END_REQ;
+
+    if (status != STATUS_SUCCESS || unix_pid == -1 || unix_tid == -1)
+        return;
+
+    current_tid = mach_thread_self();
+    mach_port_deallocate(mach_task_self(), current_tid);
+
+    if (unix_tid != current_tid)
+    {
+        static int once;
+        if (!once++) FIXME("setting other thread name not supported\n");
+        return;
+    }
+
+    len = ntdll_wcstoumbs( name->Buffer, name->Length / sizeof(WCHAR), nameA, sizeof(nameA) - 1, FALSE );
+    nameA[len] = '\0';
+    pthread_setname_np(nameA);
+#else
+    static int once;
+    if (!once++) FIXME("not implemented on this platform\n");
+#endif
+}
+
 #ifndef _WIN64
 static BOOL is_process_wow64( const CLIENT_ID *id )
 {
@@ -1852,9 +1945,9 @@ static BOOL is_process_wow64( const CLIENT_ID *id )
 NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
                                           void *data, ULONG length, ULONG *ret_len )
 {
-    NTSTATUS status;
+    unsigned int status;
 
-    TRACE("(%p,%d,%p,%x,%p)\n", handle, class, data, length, ret_len);
+    TRACE("(%p,%d,%p,%x,%p)\n", handle, class, data, (int)length, ret_len);
 
     switch (class)
     {
@@ -2084,6 +2177,17 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         if (ret_len) *ret_len = sizeof(BOOLEAN);
         return STATUS_SUCCESS;
 
+    case ThreadPriorityBoost:
+    {
+        DWORD *value = data;
+
+        if (length != sizeof(ULONG)) return STATUS_INFO_LENGTH_MISMATCH;
+        if (ret_len) *ret_len = sizeof(ULONG);
+        *value = 0;
+        return STATUS_SUCCESS;
+    }
+
+    case ThreadIdealProcessor:
     case ThreadEnableAlignmentFaultFixup:
         return STATUS_INVALID_INFO_CLASS;
 
@@ -2093,8 +2197,6 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
     case ThreadEventPair_Reusable:
     case ThreadZeroTlsCell:
     case ThreadPerformanceCount:
-    case ThreadIdealProcessor:
-    case ThreadPriorityBoost:
     case ThreadSetTlsArrayAddress:
     default:
         FIXME( "info class %d not supported yet\n", class );
@@ -2109,9 +2211,9 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
 NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
                                         const void *data, ULONG length )
 {
-    NTSTATUS status;
+    unsigned int status;
 
-    TRACE("(%p,%d,%p,%x)\n", handle, class, data, length);
+    TRACE("(%p,%d,%p,%x)\n", handle, class, data, (int)length);
 
     switch (class)
     {
@@ -2233,12 +2335,20 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
     case ThreadNameInformation:
     {
         const THREAD_NAME_INFORMATION *info = data;
+        THREAD_BASIC_INFORMATION tbi;
 
         if (length != sizeof(*info)) return STATUS_INFO_LENGTH_MISMATCH;
         if (!info) return STATUS_ACCESS_VIOLATION;
-        if (info->ThreadName.Length != info->ThreadName.MaximumLength) return STATUS_INVALID_PARAMETER;
         if (info->ThreadName.Length && !info->ThreadName.Buffer) return STATUS_ACCESS_VIOLATION;
 
+        status = NtQueryInformationThread( handle, ThreadBasicInformation, &tbi, sizeof(tbi), NULL );
+        if (handle == GetCurrentThread() || (!status && (HandleToULong(tbi.ClientId.UniqueThread) == GetCurrentThreadId())))
+            WARN_(threadname)( "Thread renamed to %s\n", debugstr_us(&info->ThreadName) );
+        else if (!status)
+            WARN_(threadname)( "Thread ID %04x renamed to %s\n", (int)HandleToULong( tbi.ClientId.UniqueThread ), debugstr_us(&info->ThreadName) );
+        else
+            WARN_(threadname)( "Thread handle %p renamed to %s\n", handle, debugstr_us(&info->ThreadName) );
+
         SERVER_START_REQ( set_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
@@ -2247,9 +2357,22 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
             status = wine_server_call( req );
         }
         SERVER_END_REQ;
+
+        set_native_thread_name( handle, &info->ThreadName );
+
         return status;
     }
 
+    case ThreadWineNativeThreadName:
+    {
+        const THREAD_NAME_INFORMATION *info = data;
+
+        if (length != sizeof(*info)) return STATUS_INFO_LENGTH_MISMATCH;
+
+        set_native_thread_name( handle, &info->ThreadName );
+        return STATUS_SUCCESS;
+    }
+
     case ThreadWow64Context:
         return set_thread_wow64_context( handle, data, length );
 
@@ -2259,6 +2382,26 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
         FIXME( "ThreadEnableAlignmentFaultFixup stub!\n" );
         return STATUS_SUCCESS;
 
+    case ThreadPowerThrottlingState:
+        if (length != sizeof(THREAD_POWER_THROTTLING_STATE)) return STATUS_INFO_LENGTH_MISMATCH;
+        if (!data) return STATUS_ACCESS_VIOLATION;
+        FIXME( "ThreadPowerThrottling stub!\n" );
+        return STATUS_SUCCESS;
+
+    case ThreadIdealProcessor:
+    {
+        const ULONG *number = data;
+
+        if (length != sizeof(*number)) return STATUS_INFO_LENGTH_MISMATCH;
+        if (*number > MAXIMUM_PROCESSORS) return STATUS_INVALID_PARAMETER;
+        FIXME( "ThreadIdealProcessor stub!\n" );
+        return STATUS_SUCCESS;
+    }
+
+    case ThreadPriorityBoost:
+        WARN("Unimplemented class ThreadPriorityBoost.\n");
+        return STATUS_SUCCESS;
+
     case ThreadBasicInformation:
     case ThreadTimes:
     case ThreadPriority:
@@ -2266,8 +2409,6 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
     case ThreadEventPair_Reusable:
     case ThreadPerformanceCount:
     case ThreadAmILastThread:
-    case ThreadIdealProcessor:
-    case ThreadPriorityBoost:
     case ThreadSetTlsArrayAddress:
     case ThreadIsIoPending:
     default:
@@ -2303,7 +2444,7 @@ ULONG WINAPI NtGetCurrentProcessorNumber(void)
                 {
                     if (thread_mask != processor_mask)
                         FIXME( "need multicore support (%d processors)\n",
-                               peb->NumberOfProcessors );
+                               (int)peb->NumberOfProcessors );
                     return processor;
                 }
             }
@@ -2321,10 +2462,10 @@ NTSTATUS WINAPI NtGetNextThread( HANDLE process, HANDLE thread, ACCESS_MASK acce
                                  ULONG flags, HANDLE *handle )
 {
     HANDLE ret_handle = 0;
-    NTSTATUS ret;
+    unsigned int ret;
 
     TRACE( "process %p, thread %p, access %#x, attributes %#x, flags %#x, handle %p.\n",
-            process, thread, access, attributes, flags, handle );
+            process, thread, (int)access, (int)attributes, (int)flags, handle );
 
     SERVER_START_REQ( get_next_thread )
     {
