@@ -40,7 +40,6 @@
 #include "shlguid.h"
 
 #include "pidl.h"
-#include "undocshell.h"
 #include "shell32_main.h"
 #include "shlwapi.h"
 #include "wine/debug.h"
@@ -48,6 +47,10 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL (shell);
 
+/* SHCreateLinks flags */
+#define SHCLF_PREFIXNAME        0x01
+#define SHCLF_CREATEONDESKTOP   0x02
+
 /***************************************************************************
  *  SHELL32_GetCustomFolderAttribute (internal function)
  *
@@ -115,7 +118,7 @@ LPCWSTR GetNextElementW (LPCWSTR pszNext, LPWSTR pszOut, DWORD dwOut)
     LPCWSTR pszTail = pszNext;
     DWORD dwCopy;
 
-    TRACE ("(%s %p 0x%08x)\n", debugstr_w(pszNext), pszOut, dwOut);
+    TRACE ("(%s %p 0x%08lx)\n", debugstr_w(pszNext), pszOut, dwOut);
 
     *pszOut = 0;
 
@@ -133,7 +136,7 @@ LPCWSTR GetNextElementW (LPCWSTR pszNext, LPWSTR pszOut, DWORD dwOut)
     else
 	pszTail = NULL;
 
-    TRACE ("--(%s %s 0x%08x %p)\n", debugstr_w (pszNext), debugstr_w (pszOut), dwOut, pszTail);
+    TRACE ("--(%s %s 0x%08lx %p)\n", debugstr_w (pszNext), debugstr_w (pszOut), dwOut, pszTail);
     return pszTail;
 }
 
@@ -167,7 +170,7 @@ HRESULT SHELL32_ParseNextElement (IShellFolder2 * psf, HWND hwndOwner, LPBC pbc,
     ILFree (*pidlInOut);
     *pidlInOut = pidlTemp;
 
-    TRACE ("-- pidl=%p ret=0x%08x\n", pidlInOut ? *pidlInOut : NULL, hr);
+    TRACE ("-- pidl=%p ret=0x%08lx\n", pidlInOut ? *pidlInOut : NULL, hr);
     return hr;
 }
 
@@ -228,7 +231,7 @@ static HRESULT SHELL32_CoCreateInitSF (LPCITEMIDLIST pidlRoot, LPCWSTR pathRoot,
 	}
 	ILFree (pidlAbsolute);
     }
-    TRACE ("-- (%p) ret=0x%08x\n", *ppvOut, hr);
+    TRACE ("-- (%p) ret=0x%08lx\n", *ppvOut, hr);
     return hr;
 }
 
@@ -305,7 +308,7 @@ HRESULT SHELL32_BindToChild (LPCITEMIDLIST pidlRoot, const CLSID *clsidChild,
         IShellFolder_Release (pSF);
     }
 
-    TRACE ("-- returning (%p) 0x%08x\n", *ppvOut, hr);
+    TRACE ("-- returning (%p) 0x%08lx\n", *ppvOut, hr);
 
     return hr;
 }
@@ -330,7 +333,7 @@ HRESULT SHELL32_GetDisplayNameOfChild (IShellFolder2 * psf,
     LPITEMIDLIST pidlFirst;
     HRESULT hr;
 
-    TRACE ("(%p)->(pidl=%p 0x%08x %p 0x%08x)\n", psf, pidl, dwFlags, szOut, dwOutLen);
+    TRACE ("(%p)->(pidl=%p 0x%08lx %p 0x%08lx)\n", psf, pidl, dwFlags, szOut, dwOutLen);
     pdump (pidl);
 
     pidlFirst = ILCloneFirst (pidl);
@@ -353,7 +356,7 @@ HRESULT SHELL32_GetDisplayNameOfChild (IShellFolder2 * psf,
     } else
 	hr = E_OUTOFMEMORY;
 
-    TRACE ("-- ret=0x%08x %s\n", hr, debugstr_w(szOut));
+    TRACE ("-- ret=0x%08lx %s\n", hr, debugstr_w(szOut));
 
     return hr;
 }
@@ -397,11 +400,11 @@ HRESULT SHELL32_GetItemAttributes (IShellFolder2 *psf, LPCITEMIDLIST pidl, LPDWO
                           SFGAO_FILESYSTEM |        /*0x40000000 */
                           SFGAO_HASSUBFOLDER;       /*0x80000000 */
     
-    TRACE ("0x%08x\n", *pdwAttributes);
+    TRACE ("0x%08lx\n", *pdwAttributes);
 
     if (*pdwAttributes & ~dwSupportedAttr)
     {
-        WARN ("attributes 0x%08x not implemented\n", (*pdwAttributes & ~dwSupportedAttr));
+        WARN ("attributes 0x%08lx not implemented\n", (*pdwAttributes & ~dwSupportedAttr));
         *pdwAttributes &= dwSupportedAttr;
     }
 
@@ -464,7 +467,7 @@ HRESULT SHELL32_GetItemAttributes (IShellFolder2 *psf, LPCITEMIDLIST pidl, LPDWO
     } else {
 	*pdwAttributes &= SFGAO_HASSUBFOLDER|SFGAO_FOLDER|SFGAO_FILESYSANCESTOR|SFGAO_DROPTARGET|SFGAO_HASPROPSHEET|SFGAO_CANRENAME|SFGAO_CANLINK;
     }
-    TRACE ("-- 0x%08x\n", *pdwAttributes);
+    TRACE ("-- 0x%08lx\n", *pdwAttributes);
     return S_OK;
 }
 
@@ -612,11 +615,145 @@ HRESULT WINAPI SHCreateLinks( HWND hWnd, LPCSTR lpszDir, LPDATAOBJECT lpDataObje
  *
  *   Added in XP.
  */
-HRESULT WINAPI SHOpenFolderAndSelectItems( PCIDLIST_ABSOLUTE pidlFolder, UINT cidl,
-                              PCUITEMID_CHILD_ARRAY *apidl, DWORD flags )
+HRESULT WINAPI SHOpenFolderAndSelectItems(PCIDLIST_ABSOLUTE pidlFolder, UINT cidl,
+                                          PCUITEMID_CHILD_ARRAY apidl, DWORD flags)
 {
-    FIXME("%p %u %p 0x%x: stub\n", pidlFolder, cidl, apidl, flags);
-    return E_NOTIMPL;
+    static const unsigned int magic = 0xe32ee32e;
+    unsigned int i, uint_flags, size, child_count = 0;
+    const ITEMIDLIST *pidl_parent, *pidl_child;
+    VARIANT var_parent, var_empty;
+    ITEMIDLIST *pidl_tmp = NULL;
+    SHELLEXECUTEINFOW sei = {0};
+    COPYDATASTRUCT cds = {0};
+    IDispatch *dispatch;
+    int timeout = 1000;
+    unsigned char *ptr;
+    IShellWindows *sw;
+    BOOL ret = FALSE;
+    HRESULT hr;
+    LONG hwnd;
+
+    TRACE("%p %u %p 0x%lx\n", pidlFolder, cidl, apidl, flags);
+
+    if (!pidlFolder)
+        return E_INVALIDARG;
+
+    if (flags & OFASI_OPENDESKTOP)
+        FIXME("Ignoring unsupported OFASI_OPENDESKTOP flag.\n");
+
+    if (flags & OFASI_EDIT && cidl > 1)
+        flags &= ~OFASI_EDIT;
+
+    hr = CoCreateInstance(&CLSID_ShellWindows, NULL, CLSCTX_LOCAL_SERVER, &IID_IShellWindows,
+                          (void **)&sw);
+    if (FAILED(hr))
+        return hr;
+
+    if (!cidl)
+    {
+        pidl_tmp = ILClone(pidlFolder);
+        ILRemoveLastID(pidl_tmp);
+        pidl_parent = pidl_tmp;
+
+        pidl_child = ILFindLastID(pidlFolder);
+        apidl = &pidl_child;
+        cidl = 1;
+    }
+    else
+    {
+        pidl_parent = pidlFolder;
+    }
+
+    /* Find the existing explorer window for the parent path. Create a new one if not present. */
+    VariantInit(&var_empty);
+    VariantInit(&var_parent);
+    size = ILGetSize(pidl_parent);
+    V_VT(&var_parent) = VT_ARRAY | VT_UI1;
+    V_ARRAY(&var_parent) = SafeArrayCreateVector(VT_UI1, 0, size);
+    memcpy(V_ARRAY(&var_parent)->pvData, pidl_parent, size);
+    hr = IShellWindows_FindWindowSW(sw, &var_parent, &var_empty, SWC_EXPLORER, &hwnd, 0, &dispatch);
+    if (hr != S_OK)
+    {
+        sei.cbSize = sizeof(sei);
+        sei.fMask = SEE_MASK_FLAG_NO_UI | SEE_MASK_IDLIST | SEE_MASK_NOASYNC | SEE_MASK_WAITFORINPUTIDLE;
+        sei.lpVerb = L"explore";
+        sei.lpIDList = (void *)pidl_parent;
+        sei.nShow = SW_NORMAL;
+        if (!ShellExecuteExW(&sei))
+        {
+            WARN("Failed to create a explorer window.\n");
+            goto done;
+        }
+
+        while (timeout > 0)
+        {
+            hr = IShellWindows_FindWindowSW(sw, &var_parent, &var_empty, SWC_EXPLORER, &hwnd, 0,
+                                            &dispatch);
+            if (hr == S_OK)
+                break;
+
+            timeout -= 100;
+            Sleep(100);
+        }
+
+        if (hr != S_OK)
+        {
+            WARN("Failed to find the explorer window.\n");
+            goto done;
+        }
+    }
+
+    /* Send WM_COPYDATA to tell explorer.exe to open windows */
+    size = sizeof(cidl) + sizeof(uint_flags);
+    for (i = 0; i < cidl; ++i)
+        size += ILGetSize(apidl[i]);
+
+    cds.dwData = magic;
+    cds.cbData = size;
+    cds.lpData = malloc(size);
+    if (!cds.lpData)
+    {
+        hr = E_OUTOFMEMORY;
+        goto done;
+    }
+
+    /* Add the count of child ITEMIDLIST, set its value at the end */
+    ptr = (unsigned char *)cds.lpData + sizeof(cidl);
+
+    /* Add flags. Have to use unsigned int because DWORD may have a different size */
+    uint_flags = flags;
+    memcpy(ptr, &uint_flags, sizeof(uint_flags));
+    ptr += sizeof(uint_flags);
+
+    /* Add child ITEMIDLIST */
+    for (i = 0; i < cidl; ++i)
+    {
+        if (apidl != &pidl_child)
+            pidl_child = ILFindChild(pidl_parent, apidl[i]);
+
+        if (pidl_child)
+        {
+            size = ILGetSize(pidl_child);
+            memcpy(ptr, pidl_child, size);
+            ptr += size;
+            ++child_count;
+        }
+    }
+
+    /* Set the count of child ITEMIDLIST */
+    memcpy(cds.lpData, &child_count, sizeof(child_count));
+
+    SetForegroundWindow(GetAncestor((HWND)(LONG_PTR)hwnd, GA_ROOT));
+    ret = SendMessageW((HWND)(LONG_PTR)hwnd, WM_COPYDATA, 0, (LPARAM)&cds);
+    hr = ret ? S_OK : E_FAIL;
+
+done:
+    free(cds.lpData);
+    VariantClear(&var_parent);
+    if (pidl_tmp)
+        ILFree(pidl_tmp);
+    IShellWindows_Release(sw);
+    return hr;
 }
 
 /***********************************************************************
@@ -653,7 +790,7 @@ HRESULT WINAPI SHGetSetFolderCustomSettings( LPSHFOLDERCUSTOMSETTINGS fcs, PCWST
         }
     }
     else
-        FIXME("%p %s 0x%x: stub\n", fcs, debugstr_w(path), flag);
+        FIXME("%p %s 0x%lx: stub\n", fcs, debugstr_w(path), flag);
 
     return hr;
 }
