@@ -74,7 +74,7 @@ static ULONG WINAPI mqr_AddRef(IWICMetadataQueryReader *iface)
 {
     QueryReader *This = impl_from_IWICMetadataQueryReader(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
-    TRACE("(%p) refcount=%u\n", This, ref);
+    TRACE("(%p) refcount=%lu\n", This, ref);
     return ref;
 }
 
@@ -82,12 +82,12 @@ static ULONG WINAPI mqr_Release(IWICMetadataQueryReader *iface)
 {
     QueryReader *This = impl_from_IWICMetadataQueryReader(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
-    TRACE("(%p) refcount=%u\n", This, ref);
+    TRACE("(%p) refcount=%lu\n", This, ref);
     if (!ref)
     {
         IWICMetadataBlockReader_Release(This->block);
-        HeapFree(GetProcessHeap(), 0, This->root);
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This->root);
+        free(This);
     }
     return ref;
 }
@@ -210,7 +210,7 @@ static HRESULT get_token(struct string_t *elem, PROPVARIANT *id, PROPVARIANT *sc
         hr = get_token(&next_elem, id, schema, idx);
         if (hr != S_OK)
         {
-            TRACE("get_token error %#x\n", hr);
+            TRACE("get_token error %#lx\n", hr);
             return hr;
         }
         elem->len = (end - start) + next_elem.len;
@@ -286,7 +286,7 @@ static HRESULT get_token(struct string_t *elem, PROPVARIANT *id, PROPVARIANT *sc
             hr = get_token(&next_elem, &next_id, &next_schema, &next_idx);
             if (hr != S_OK)
             {
-                TRACE("get_token error %#x\n", hr);
+                TRACE("get_token error %#lx\n", hr);
                 return hr;
             }
             elem->len = (end - start + 1) + next_elem.len;
@@ -346,7 +346,7 @@ static HRESULT get_token(struct string_t *elem, PROPVARIANT *id, PROPVARIANT *sc
         hr = get_token(&next_elem, &next_id, &next_schema, &next_idx);
         if (hr != S_OK)
         {
-            TRACE("get_token error %#x\n", hr);
+            TRACE("get_token error %#lx\n", hr);
             PropVariantClear(id);
             PropVariantClear(schema);
             return hr;
@@ -461,7 +461,7 @@ static HRESULT WINAPI mqr_GetMetadataByName(IWICMetadataQueryReader *iface, LPCW
 
     len = lstrlenW(query) + 1;
     if (This->root) len += lstrlenW(This->root);
-    full_query = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+    full_query = malloc(len * sizeof(WCHAR));
     full_query[0] = 0;
     if (This->root)
         lstrcpyW(full_query, This->root);
@@ -491,7 +491,7 @@ static HRESULT WINAPI mqr_GetMetadataByName(IWICMetadataQueryReader *iface, LPCW
         hr = get_token(&elem, &tk_id, &tk_schema, &index);
         if (hr != S_OK)
         {
-            WARN("get_token error %#x\n", hr);
+            WARN("get_token error %#lx\n", hr);
             break;
         }
         TRACE("parsed %d characters: %s, index %d\n", elem.len, wine_dbgstr_wn(elem.str, elem.len), index);
@@ -591,7 +591,7 @@ static HRESULT WINAPI mqr_GetMetadataByName(IWICMetadataQueryReader *iface, LPCW
     else
         PropVariantClear(&new_value);
 
-    HeapFree(GetProcessHeap(), 0, full_query);
+    free(full_query);
 
     return hr;
 }
@@ -631,7 +631,7 @@ static ULONG WINAPI string_enumerator_AddRef(IEnumString *iface)
     struct string_enumerator *this = impl_from_IEnumString(iface);
     ULONG ref = InterlockedIncrement(&this->ref);
 
-    TRACE("iface %p, ref %u.\n", iface, ref);
+    TRACE("iface %p, ref %lu.\n", iface, ref);
 
     return ref;
 }
@@ -641,7 +641,7 @@ static ULONG WINAPI string_enumerator_Release(IEnumString *iface)
     struct string_enumerator *this = impl_from_IEnumString(iface);
     ULONG ref = InterlockedDecrement(&this->ref);
 
-    TRACE("iface %p, ref %u.\n", iface, ref);
+    TRACE("iface %p, ref %lu.\n", iface, ref);
 
     if (!ref)
         free(this);
@@ -651,7 +651,7 @@ static ULONG WINAPI string_enumerator_Release(IEnumString *iface)
 
 static HRESULT WINAPI string_enumerator_Next(IEnumString *iface, ULONG count, LPOLESTR *strings, ULONG *ret)
 {
-    FIXME("iface %p, count %u, strings %p, ret %p stub.\n", iface, count, strings, ret);
+    FIXME("iface %p, count %lu, strings %p, ret %p stub.\n", iface, count, strings, ret);
 
     if (!strings || !ret)
         return E_INVALIDARG;
@@ -669,7 +669,7 @@ static HRESULT WINAPI string_enumerator_Reset(IEnumString *iface)
 
 static HRESULT WINAPI string_enumerator_Skip(IEnumString *iface, ULONG count)
 {
-    FIXME("iface %p, count %u stub.\n", iface, count);
+    FIXME("iface %p, count %lu stub.\n", iface, count);
 
     return count ? S_FALSE : S_OK;
 }
@@ -730,7 +730,7 @@ HRESULT MetadataQueryReader_CreateInstance(IWICMetadataBlockReader *mbr, const W
 {
     QueryReader *obj;
 
-    obj = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*obj));
+    obj = calloc(1, sizeof(*obj));
     if (!obj)
         return E_OUTOFMEMORY;
 
@@ -740,7 +740,7 @@ HRESULT MetadataQueryReader_CreateInstance(IWICMetadataBlockReader *mbr, const W
     IWICMetadataBlockReader_AddRef(mbr);
     obj->block = mbr;
 
-    obj->root = root ? heap_strdupW(root) : NULL;
+    obj->root = wcsdup(root);
 
     *out = &obj->IWICMetadataQueryReader_iface;
 
@@ -789,7 +789,7 @@ static ULONG WINAPI mqw_AddRef(IWICMetadataQueryWriter *iface)
     QueryWriter *writer = impl_from_IWICMetadataQueryWriter(iface);
     ULONG ref = InterlockedIncrement(&writer->ref);
 
-    TRACE("writer %p, refcount=%u\n", writer, ref);
+    TRACE("writer %p, refcount=%lu\n", writer, ref);
 
     return ref;
 }
@@ -799,13 +799,13 @@ static ULONG WINAPI mqw_Release(IWICMetadataQueryWriter *iface)
     QueryWriter *writer = impl_from_IWICMetadataQueryWriter(iface);
     ULONG ref = InterlockedDecrement(&writer->ref);
 
-    TRACE("writer %p, refcount=%u.\n", writer, ref);
+    TRACE("writer %p, refcount=%lu.\n", writer, ref);
 
     if (!ref)
     {
         IWICMetadataBlockWriter_Release(writer->block);
-        HeapFree(GetProcessHeap(), 0, writer->root);
-        HeapFree(GetProcessHeap(), 0, writer);
+        free(writer->root);
+        free(writer);
     }
     return ref;
 }
@@ -870,7 +870,7 @@ HRESULT MetadataQueryWriter_CreateInstance(IWICMetadataBlockWriter *mbw, const W
 {
     QueryWriter *obj;
 
-    obj = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*obj));
+    obj = calloc(1, sizeof(*obj));
     if (!obj)
         return E_OUTOFMEMORY;
 
@@ -880,7 +880,7 @@ HRESULT MetadataQueryWriter_CreateInstance(IWICMetadataBlockWriter *mbw, const W
     IWICMetadataBlockWriter_AddRef(mbw);
     obj->block = mbw;
 
-    obj->root = root ? heap_strdupW(root) : NULL;
+    obj->root = wcsdup(root);
 
     *out = &obj->IWICMetadataQueryWriter_iface;
 
