@@ -17,30 +17,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include "wined3d_private.h"
-
-WINE_DECLARE_DEBUG_CHANNEL(winediag);
-
-#ifdef SONAME_LIBVKD3D_SHADER
-
-#define VKD3D_SHADER_NO_PROTOTYPES
+#define LIBVKD3D_SHADER_SOURCE
 #include <vkd3d_shader.h>
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d_shader);
 
-static PFN_vkd3d_shader_compile vkd3d_shader_compile;
-static PFN_vkd3d_shader_free_messages vkd3d_shader_free_messages;
-static PFN_vkd3d_shader_free_scan_descriptor_info vkd3d_shader_free_scan_descriptor_info;
-static PFN_vkd3d_shader_free_shader_code vkd3d_shader_free_shader_code;
-static PFN_vkd3d_shader_get_version vkd3d_shader_get_version;
-static PFN_vkd3d_shader_scan vkd3d_shader_scan;
-
 static const struct wined3d_shader_backend_ops spirv_shader_backend_vk;
 
-static void *vkd3d_shader_handle;
+static const struct vkd3d_shader_compile_option spirv_compile_options[] =
+{
+    {VKD3D_SHADER_COMPILE_OPTION_API_VERSION, VKD3D_SHADER_API_VERSION_1_3},
+};
 
 struct shader_spirv_resource_bindings
 {
@@ -74,6 +62,7 @@ struct shader_spirv_compile_arguments
         {
             uint32_t alpha_swizzle;
             unsigned int sample_count;
+            bool dual_source_blending;
         } fs;
     } u;
 };
@@ -118,57 +107,6 @@ struct wined3d_shader_spirv_shader_interface
     struct vkd3d_shader_transform_feedback_info xfb_info;
 };
 
-static bool wined3d_load_vkd3d_shader_functions(void *vkd3d_shader_handle)
-{
-#define LOAD_FUNCPTR(f) if (!(f = dlsym(vkd3d_shader_handle, #f))) return false;
-    LOAD_FUNCPTR(vkd3d_shader_compile)
-    LOAD_FUNCPTR(vkd3d_shader_free_messages)
-    LOAD_FUNCPTR(vkd3d_shader_free_scan_descriptor_info)
-    LOAD_FUNCPTR(vkd3d_shader_free_shader_code)
-    LOAD_FUNCPTR(vkd3d_shader_get_version)
-    LOAD_FUNCPTR(vkd3d_shader_scan)
-#undef LOAD_FUNCPTR
-
-    return true;
-}
-
-static void wined3d_unload_vkd3d_shader(void)
-{
-    if (vkd3d_shader_handle)
-    {
-        dlclose(vkd3d_shader_handle);
-        vkd3d_shader_handle = NULL;
-    }
-}
-
-static BOOL WINAPI wined3d_init_vkd3d_once(INIT_ONCE *once, void *param, void **context)
-{
-    TRACE("Loading vkd3d-shader %s.\n", SONAME_LIBVKD3D_SHADER);
-
-    if ((vkd3d_shader_handle = dlopen(SONAME_LIBVKD3D_SHADER, RTLD_NOW)))
-    {
-        if (!wined3d_load_vkd3d_shader_functions(vkd3d_shader_handle))
-        {
-            ERR_(winediag)("Failed to load libvkd3d-shader functions.\n");
-            wined3d_unload_vkd3d_shader();
-        }
-        TRACE("Using %s.\n", vkd3d_shader_get_version(NULL, NULL));
-    }
-    else
-    {
-        ERR_(winediag)("Failed to load libvkd3d-shader.\n");
-    }
-
-    return TRUE;
-}
-
-static bool wined3d_init_vkd3d(void)
-{
-    static INIT_ONCE init_once = INIT_ONCE_STATIC_INIT;
-    InitOnceExecuteOnce(&init_once, wined3d_init_vkd3d_once, NULL, NULL);
-    return !!vkd3d_shader_handle;
-}
-
 static enum vkd3d_shader_visibility vkd3d_shader_visibility_from_wined3d(enum wined3d_shader_type shader_type)
 {
     switch (shader_type)
@@ -215,6 +153,7 @@ static void shader_spirv_compile_arguments_init(struct shader_spirv_compile_argu
                     args->u.fs.alpha_swizzle |= 1u << i;
             }
             args->u.fs.sample_count = sample_count;
+            args->u.fs.dual_source_blending = state->blend_state && state->blend_state->dual_source;
             break;
 
         default:
@@ -245,6 +184,8 @@ static void shader_spirv_init_compile_args(struct wined3d_shader_spirv_compile_a
         shader_parameter->data_type = VKD3D_SHADER_PARAMETER_DATA_TYPE_UINT32;
         shader_parameter->u.immediate_constant.u.u32 = compile_args->u.fs.sample_count;
 
+        args->spirv_target.dual_source_blending = compile_args->u.fs.dual_source_blending;
+
         args->spirv_target.parameter_count = 1;
         args->spirv_target.parameters = shader_parameter;
 
@@ -278,8 +219,7 @@ static const char *get_line(const char **ptr)
 }
 
 static void shader_spirv_init_shader_interface_vk(struct wined3d_shader_spirv_shader_interface *iface,
-        struct wined3d_shader *shader, const struct shader_spirv_resource_bindings *b,
-        const struct wined3d_stream_output_desc *so_desc)
+        const struct shader_spirv_resource_bindings *b, const struct wined3d_stream_output_desc *so_desc)
 {
     memset(iface, 0, sizeof(*iface));
     iface->vkd3d_interface.type = VKD3D_SHADER_STRUCTURE_TYPE_INTERFACE_INFO;
@@ -304,16 +244,16 @@ static void shader_spirv_init_shader_interface_vk(struct wined3d_shader_spirv_sh
     iface->vkd3d_interface.uav_counter_count = b->uav_counter_count;
 }
 
-static VkShaderModule shader_spirv_compile(struct wined3d_context_vk *context_vk,
-        struct wined3d_shader *shader, const struct shader_spirv_compile_arguments *args,
-        const struct shader_spirv_resource_bindings *bindings, const struct wined3d_stream_output_desc *so_desc)
+static VkShaderModule shader_spirv_compile_shader(struct wined3d_context_vk *context_vk,
+        const struct wined3d_shader_desc *shader_desc, enum wined3d_shader_type shader_type,
+        const struct shader_spirv_compile_arguments *args, const struct shader_spirv_resource_bindings *bindings,
+        const struct wined3d_stream_output_desc *so_desc)
 {
     struct wined3d_shader_spirv_compile_args compile_args;
     struct wined3d_shader_spirv_shader_interface iface;
+    VkShaderModuleCreateInfo shader_create_info;
     struct vkd3d_shader_compile_info info;
     const struct wined3d_vk_info *vk_info;
-    enum wined3d_shader_type shader_type;
-    VkShaderModuleCreateInfo shader_desc;
     struct wined3d_device_vk *device_vk;
     struct vkd3d_shader_code spirv;
     VkShaderModule module;
@@ -321,19 +261,18 @@ static VkShaderModule shader_spirv_compile(struct wined3d_context_vk *context_vk
     VkResult vr;
     int ret;
 
-    shader_spirv_init_shader_interface_vk(&iface, shader, bindings, so_desc);
-    shader_type = shader->reg_maps.shader_version.type;
+    shader_spirv_init_shader_interface_vk(&iface, bindings, so_desc);
     shader_spirv_init_compile_args(&compile_args, &iface.vkd3d_interface,
             VKD3D_SHADER_SPIRV_ENVIRONMENT_VULKAN_1_0, shader_type, args);
 
     info.type = VKD3D_SHADER_STRUCTURE_TYPE_COMPILE_INFO;
     info.next = &compile_args.spirv_target;
-    info.source.code = shader->byte_code;
-    info.source.size = shader->byte_code_size;
+    info.source.code = shader_desc->byte_code;
+    info.source.size = shader_desc->byte_code_size;
     info.source_type = VKD3D_SHADER_SOURCE_DXBC_TPF;
     info.target_type = VKD3D_SHADER_TARGET_SPIRV_BINARY;
-    info.options = NULL;
-    info.option_count = 0;
+    info.options = spirv_compile_options;
+    info.option_count = ARRAY_SIZE(spirv_compile_options);
     info.log_level = VKD3D_SHADER_LOG_WARNING;
     info.source_name = NULL;
 
@@ -361,12 +300,12 @@ static VkShaderModule shader_spirv_compile(struct wined3d_context_vk *context_vk
     device_vk = wined3d_device_vk(context_vk->c.device);
     vk_info = &device_vk->vk_info;
 
-    shader_desc.sType = VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO;
-    shader_desc.pNext = NULL;
-    shader_desc.flags = 0;
-    shader_desc.codeSize = spirv.size;
-    shader_desc.pCode = spirv.code;
-    if ((vr = VK_CALL(vkCreateShaderModule(device_vk->vk_device, &shader_desc, NULL, &module))) < 0)
+    shader_create_info.sType = VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO;
+    shader_create_info.pNext = NULL;
+    shader_create_info.flags = 0;
+    shader_create_info.codeSize = spirv.size;
+    shader_create_info.pCode = spirv.code;
+    if ((vr = VK_CALL(vkCreateShaderModule(device_vk->vk_device, &shader_create_info, NULL, &module))) < 0)
     {
         vkd3d_shader_free_shader_code(&spirv);
         WARN("Failed to create Vulkan shader module, vr %s.\n", wined3d_debug_vkresult(vr));
@@ -388,6 +327,7 @@ static struct shader_spirv_graphics_program_variant_vk *shader_spirv_find_graphi
     const struct wined3d_stream_output_desc *so_desc = NULL;
     struct shader_spirv_graphics_program_vk *program_vk;
     struct shader_spirv_compile_arguments args;
+    struct wined3d_shader_desc shader_desc;
     size_t variant_count, i;
 
     shader_spirv_compile_arguments_init(&args, &context_vk->c, shader, state, context_vk->sample_count);
@@ -413,7 +353,12 @@ static struct shader_spirv_graphics_program_variant_vk *shader_spirv_find_graphi
     variant_vk = &program_vk->variants[variant_count];
     variant_vk->compile_args = args;
     variant_vk->binding_base = binding_base;
-    if (!(variant_vk->vk_module = shader_spirv_compile(context_vk, shader, &args, bindings, so_desc)))
+
+    shader_desc.byte_code = shader->byte_code;
+    shader_desc.byte_code_size = shader->byte_code_size;
+
+    if (!(variant_vk->vk_module = shader_spirv_compile_shader(context_vk, &shader_desc, shader_type, &args,
+            bindings, so_desc)))
         return NULL;
     ++program_vk->variant_count;
 
@@ -429,6 +374,7 @@ static struct shader_spirv_compute_program_vk *shader_spirv_find_compute_program
     struct shader_spirv_compute_program_vk *program;
     struct wined3d_pipeline_layout_vk *layout;
     VkComputePipelineCreateInfo pipeline_info;
+    struct wined3d_shader_desc shader_desc;
     VkResult vr;
 
     if (!(program = shader->backend_data))
@@ -437,7 +383,11 @@ static struct shader_spirv_compute_program_vk *shader_spirv_find_compute_program
     if (program->vk_module)
         return program;
 
-    if (!(program->vk_module = shader_spirv_compile(context_vk, shader, NULL, bindings, NULL)))
+    shader_desc.byte_code = shader->byte_code;
+    shader_desc.byte_code_size = shader->byte_code_size;
+
+    if (!(program->vk_module = shader_spirv_compile_shader(context_vk, &shader_desc, WINED3D_SHADER_TYPE_COMPUTE,
+            NULL, bindings, NULL)))
         return NULL;
 
     if (!(layout = wined3d_context_vk_get_pipeline_layout(context_vk,
@@ -651,6 +601,8 @@ static enum wined3d_data_type wined3d_data_type_from_vkd3d(enum vkd3d_shader_res
             return WINED3D_DATA_UINT;
         case VKD3D_SHADER_RESOURCE_DATA_FLOAT:
             return WINED3D_DATA_FLOAT;
+        case VKD3D_SHADER_RESOURCE_DATA_MIXED:
+            return WINED3D_DATA_UINT;
         default:
             FIXME("Unhandled resource data type %#x.\n", t);
             return WINED3D_DATA_FLOAT;
@@ -661,7 +613,7 @@ static bool shader_spirv_resource_bindings_init(struct shader_spirv_resource_bin
         struct wined3d_shader_resource_bindings *wined3d_bindings,
         const struct wined3d_state *state, uint32_t shader_mask)
 {
-    struct vkd3d_shader_scan_descriptor_info *descriptor_info;
+    const struct vkd3d_shader_scan_descriptor_info *descriptor_info;
     enum wined3d_shader_descriptor_type wined3d_type;
     enum vkd3d_shader_visibility shader_visibility;
     enum wined3d_shader_type shader_type;
@@ -700,7 +652,7 @@ static bool shader_spirv_resource_bindings_init(struct shader_spirv_resource_bin
 
         for (i = 0; i < descriptor_info->descriptor_count; ++i)
         {
-            struct vkd3d_shader_descriptor_info *d = &descriptor_info->descriptors[i];
+            const struct vkd3d_shader_descriptor_info *d = &descriptor_info->descriptors[i];
             uint32_t flags;
 
             if (d->register_space)
@@ -758,8 +710,8 @@ static void shader_spirv_scan_shader(struct wined3d_shader *shader,
     info.source.size = shader->byte_code_size;
     info.source_type = VKD3D_SHADER_SOURCE_DXBC_TPF;
     info.target_type = VKD3D_SHADER_TARGET_SPIRV_BINARY;
-    info.options = NULL;
-    info.option_count = 0;
+    info.options = spirv_compile_options;
+    info.option_count = ARRAY_SIZE(spirv_compile_options);
     info.log_level = VKD3D_SHADER_LOG_WARNING;
     info.source_name = NULL;
 
@@ -985,10 +937,11 @@ static void shader_spirv_destroy_compute_vk(struct wined3d_shader *shader)
 {
     struct wined3d_device_vk *device_vk = wined3d_device_vk(shader->device);
     struct shader_spirv_compute_program_vk *program = shader->backend_data;
+    struct wined3d_context_vk *context_vk = &device_vk->context_vk;
     struct wined3d_vk_info *vk_info = &device_vk->vk_info;
 
     shader_spirv_invalidate_contexts_compute_program(&device_vk->d, program);
-    VK_CALL(vkDestroyPipeline(device_vk->vk_device, program->vk_pipeline, NULL));
+    wined3d_context_vk_destroy_vk_pipeline(context_vk, program->vk_pipeline, context_vk->current_command_buffer.id);
     VK_CALL(vkDestroyShaderModule(device_vk->vk_device, program->vk_module, NULL));
     vkd3d_shader_free_scan_descriptor_info(&program->descriptor_info);
     shader->backend_data = NULL;
@@ -1019,6 +972,7 @@ static void shader_spirv_destroy(struct wined3d_shader *shader)
         shader_spirv_invalidate_contexts_graphics_program_variant(&device_vk->d, variant_vk);
         VK_CALL(vkDestroyShaderModule(device_vk->vk_device, variant_vk->vk_module, NULL));
     }
+    heap_free(program_vk->variants);
     vkd3d_shader_free_scan_descriptor_info(&program_vk->descriptor_info);
 
     shader->backend_data = NULL;
@@ -1116,6 +1070,13 @@ static BOOL shader_spirv_has_ffp_proj_control(void *shader_priv)
     return priv->ffp_proj_control;
 }
 
+static uint64_t shader_spirv_compile(struct wined3d_context *context, const struct wined3d_shader_desc *shader_desc,
+        enum wined3d_shader_type shader_type)
+{
+    struct shader_spirv_resource_bindings bindings = {0};
+    return (uint64_t)shader_spirv_compile_shader(wined3d_context_vk(context), shader_desc, shader_type, NULL, &bindings, NULL);
+}
+
 static const struct wined3d_shader_backend_ops spirv_shader_backend_vk =
 {
     .shader_handle_instruction = shader_spirv_handle_instruction,
@@ -1135,21 +1096,16 @@ static const struct wined3d_shader_backend_ops spirv_shader_backend_vk =
     .shader_get_caps = shader_spirv_get_caps,
     .shader_color_fixup_supported = shader_spirv_color_fixup_supported,
     .shader_has_ffp_proj_control = shader_spirv_has_ffp_proj_control,
+    .shader_compile = shader_spirv_compile,
 };
 
 const struct wined3d_shader_backend_ops *wined3d_spirv_shader_backend_init_vk(void)
 {
-    if (!wined3d_init_vkd3d())
-        return NULL;
+    TRACE("Using %s.\n", vkd3d_shader_get_version(NULL, NULL));
 
     return &spirv_shader_backend_vk;
 }
 
-void wined3d_spirv_shader_backend_cleanup(void)
-{
-    wined3d_unload_vkd3d_shader();
-}
-
 static void spirv_vertex_pipe_vk_vp_enable(const struct wined3d_context *context, BOOL enable)
 {
     /* Nothing to do. */
@@ -1158,9 +1114,11 @@ static void spirv_vertex_pipe_vk_vp_enable(const struct wined3d_context *context
 static void spirv_vertex_pipe_vk_vp_get_caps(const struct wined3d_adapter *adapter, struct wined3d_vertex_caps *caps)
 {
     memset(caps, 0, sizeof(*caps));
+    caps->xyzrhw = TRUE;
+    caps->ffp_generic_attributes = TRUE;
 }
 
-static uint32_t spirv_vertex_pipe_vk_vp_get_emul_mask(const struct wined3d_gl_info *gl_info)
+static unsigned int spirv_vertex_pipe_vk_vp_get_emul_mask(const struct wined3d_gl_info *gl_info)
 {
     return 0;
 }
@@ -1236,7 +1194,7 @@ static void spirv_fragment_pipe_vk_fp_get_caps(const struct wined3d_adapter *ada
     memset(caps, 0, sizeof(*caps));
 }
 
-static uint32_t spirv_fragment_pipe_vk_fp_get_emul_mask(const struct wined3d_gl_info *gl_info)
+static unsigned int spirv_fragment_pipe_vk_fp_get_emul_mask(const struct wined3d_gl_info *gl_info)
 {
     return 0;
 }
@@ -1307,27 +1265,3 @@ const struct wined3d_fragment_pipe_ops *wined3d_spirv_fragment_pipe_init_vk(void
 {
     return &spirv_fragment_pipe_vk;
 }
-
-#else
-
-const struct wined3d_shader_backend_ops *wined3d_spirv_shader_backend_init_vk(void)
-{
-    ERR_(winediag)("Wine was built without libvkd3d-shader support.\n");
-    return NULL;
-}
-
-void wined3d_spirv_shader_backend_cleanup(void)
-{
-}
-
-const struct wined3d_vertex_pipe_ops *wined3d_spirv_vertex_pipe_init_vk(void)
-{
-    return &none_vertex_pipe;
-}
-
-const struct wined3d_fragment_pipe_ops *wined3d_spirv_fragment_pipe_init_vk(void)
-{
-    return &none_fragment_pipe;
-}
-
-#endif /* defined(SONAME_LIBVKD3D_SHADER) */
