@@ -69,6 +69,7 @@ struct edit_line
     unsigned int           end_offset;          /* offset of the last written char */
     unsigned int           home_x;              /* home position */
     unsigned int           home_y;
+    unsigned int           ctrl_mask;           /* mask for ctrl characters for completion */
 };
 
 struct console
@@ -77,6 +78,8 @@ struct console
     unsigned int           mode;                /* input mode */
     struct screen_buffer  *active;              /* active screen buffer */
     int                    is_unix;             /* UNIX terminal mode */
+    int                    use_relative_cursor; /* use relative cursor positioning */
+    int                    no_window;           /* don't create console window */
     INPUT_RECORD          *records;             /* input records */
     unsigned int           record_count;        /* number of input records */
     unsigned int           record_size;         /* size of input records buffer */
@@ -87,6 +90,7 @@ struct console
     unsigned int           read_ioctl;          /* current read ioctl */
     size_t                 pending_read;        /* size of pending read buffer */
     struct edit_line       edit_line;           /* edit line context */
+    unsigned int           key_state;
     struct console_window *window;
     WCHAR                 *title;               /* console title */
     struct history_line  **history;             /* lines history */
@@ -130,17 +134,21 @@ struct screen_buffer
     struct wine_rb_entry   entry;               /* map entry */
 };
 
-BOOL init_window( struct console *console );
-void init_message_window( struct console *console );
-void update_window_region( struct console *console, const RECT *update );
-void update_window_config( struct console *console, BOOL delay );
-
+/* conhost.c */
 NTSTATUS write_console_input( struct console *console, const INPUT_RECORD *records,
                               unsigned int count, BOOL flush );
 
 void notify_screen_buffer_size( struct screen_buffer *screen_buffer );
 NTSTATUS change_screen_buffer_size( struct screen_buffer *screen_buffer, int new_width, int new_height );
 
+/* window.c */
+void update_console_font( struct console *console, const WCHAR *face_name, size_t face_name_size,
+                          unsigned int height, unsigned int weight );
+BOOL init_window( struct console *console );
+void init_message_window( struct console *console );
+void update_window_region( struct console *console, const RECT *update );
+void update_window_config( struct console *console, BOOL delay );
+
 static inline void empty_update_rect( struct screen_buffer *screen_buffer, RECT *rect )
 {
     SetRect( rect, screen_buffer->width, screen_buffer->height, 0, 0 );
@@ -178,7 +186,6 @@ static inline unsigned int get_bounded_cursor_x( struct screen_buffer *screen_bu
 #define IDD_OPTION              0x0100
 #define IDD_FONT                0x0200
 #define IDD_CONFIG              0x0300
-#define IDD_SAVE_SETTINGS       0x0400
 
 /* dialog boxes controls */
 #define IDC_OPT_CURSOR_SMALL    0x0101
@@ -209,6 +216,3 @@ static inline unsigned int get_bounded_cursor_x( struct screen_buffer *screen_bu
 #define IDC_CNF_WIN_HEIGHT_UD   0x0308
 #define IDC_CNF_CLOSE_EXIT      0x0309
 #define IDC_CNF_EDITION_MODE    0x030a
-
-#define IDC_SAV_SAVE            0x0401
-#define IDC_SAV_SESSION         0x0402
