@@ -104,7 +104,7 @@ static enum dllmode string_to_mode(const WCHAR *in)
     enum dllmode res;
 
     len = wcslen(in);
-    out = HeapAlloc(GetProcessHeap(), 0, (len + 1) * sizeof(WCHAR));
+    out = malloc((len + 1) * sizeof(WCHAR));
 
     /* remove the spaces */
     for (i = j = 0; i <= len; ++i) {
@@ -121,7 +121,7 @@ static enum dllmode string_to_mode(const WCHAR *in)
     if (wcscmp(out, L"native") == 0) res = NATIVE;
     if (wcscmp(out, L"") == 0) res = DISABLE;
 
-    HeapFree(GetProcessHeap(), 0, out);
+    free(out);
     return res;
 }
 
@@ -223,8 +223,6 @@ static BOOL show_dll_in_list( const WCHAR *name )
         /* skip exes */
         if (!wcscmp( ext, L".exe" )) return FALSE;
     }
-    /* skip api set placeholders */
-    if (!wcsncmp( name, L"api-ms-", 7 ) || !wcsncmp( name, L"ext-ms-", 7 )) return FALSE;
     /* skip dlls that should always be builtin */
     return !is_builtin_only( name );
 }
@@ -246,22 +244,22 @@ static void clear_settings(HWND dialog)
         struct dll *dll = (struct dll *) SendDlgItemMessageW(dialog, IDC_DLLS_LIST, LB_GETITEMDATA, 0, 0);
 
         SendDlgItemMessageW(dialog, IDC_DLLS_LIST, LB_DELETESTRING, 0, 0);
-        HeapFree(GetProcessHeap(), 0, dll->name);
-        HeapFree(GetProcessHeap(), 0, dll);
+        free(dll->name);
+        free(dll);
     }
 }
 
 /* load the list of available libraries from a given dir */
 static void load_library_list_from_dir( HWND dialog, const WCHAR *dir_path, int check_subdirs )
 {
-    static const WCHAR * const ext[] = { L".dll", L".dll.so", L".so", L"" };
+    static const WCHAR * const ext[] = { L".dll", L"", L".dll.so", L".so" };
     WCHAR *buffer, *p, name[256];
     unsigned int i;
     HANDLE handle;
     WIN32_FIND_DATAW data;
+    ULONG maxlen = wcslen(dir_path) + wcslen(pe_dir) + 10 + 2 * ARRAY_SIZE(name);
 
-    buffer = HeapAlloc( GetProcessHeap(), 0, (wcslen(dir_path) + 10) * sizeof(WCHAR) + 2 * sizeof(name) );
-
+    buffer = malloc( maxlen * sizeof(WCHAR) );
     wcscpy( buffer, dir_path );
     wcscat( buffer, L"\\*" );
     buffer[1] = '\\';  /* change \??\ to \\?\ */
@@ -269,7 +267,7 @@ static void load_library_list_from_dir( HWND dialog, const WCHAR *dir_path, int
 
     if ((handle = FindFirstFileW( buffer, &data )) == INVALID_HANDLE_VALUE)
     {
-        HeapFree( GetProcessHeap(), 0, buffer );
+        free( buffer );
         return;
     }
 
@@ -284,7 +282,9 @@ static void load_library_list_from_dir( HWND dialog, const WCHAR *dir_path, int
             if (!show_dll_in_list( data.cFileName )) continue;
             for (i = 0; i < ARRAY_SIZE( ext ); i++)
             {
-                swprintf( p, 2 * ARRAY_SIZE(name) + 10, L"%s\\%s%s", data.cFileName, data.cFileName, ext[i] );
+                if (!ext[i][0] && !wcschr( data.cFileName, '.' )) continue;
+                swprintf( p, buffer + maxlen - p, L"%s%s\\%s%s", data.cFileName,
+                          i > 1 ? L"" : pe_dir, data.cFileName, ext[i] );
                 if (GetFileAttributesW( buffer ) != INVALID_FILE_ATTRIBUTES)
                 {
                     SendDlgItemMessageW( dialog, IDC_DLLCOMBO, CB_ADDSTRING, 0, (LPARAM)data.cFileName );
@@ -310,7 +310,7 @@ static void load_library_list_from_dir( HWND dialog, const WCHAR *dir_path, int
     } while (FindNextFileW( handle, &data ));
 
     FindClose( handle );
-    HeapFree( GetProcessHeap(), 0, buffer );
+    free( buffer );
 }
 
 /* load the list of available libraries */
@@ -322,11 +322,11 @@ static void load_library_list( HWND dialog )
 
     if (GetEnvironmentVariableW( L"WINEBUILDDIR", path, MAX_PATH ))
     {
-        WCHAR *dir = HeapAlloc( GetProcessHeap(), 0, wcslen(path) * sizeof(WCHAR) + sizeof(L"\\dlls") );
+        WCHAR *dir = malloc( wcslen(path) * sizeof(WCHAR) + sizeof(L"\\dlls") );
         wcscpy( dir, path );
         wcscat( dir, L"\\dlls" );
         load_library_list_from_dir( dialog, dir, TRUE );
-        HeapFree( GetProcessHeap(), 0, dir );
+        free( dir );
     }
 
     for (;;)
@@ -374,7 +374,7 @@ static void load_library_settings(HWND dialog)
         set_controls_from_selection(dialog);
         disable(IDC_DLLS_EDITDLL);
         disable(IDC_DLLS_REMOVEDLL);
-        HeapFree(GetProcessHeap(), 0, overrides);
+        free(overrides);
         return;
     }
 
@@ -393,22 +393,22 @@ static void load_library_settings(HWND dialog)
         label = mode_to_label(string_to_mode(value));
 
         len = wcslen(*p) + 2 + wcslen(label) + 2;
-        str = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR) );
+        str = malloc(len * sizeof(WCHAR));
         swprintf( str, len, L"%s (%s)", *p, label );
 
-        dll = HeapAlloc(GetProcessHeap(), 0, sizeof(struct dll));
+        dll = malloc(sizeof(struct dll));
         dll->name = *p;
         dll->mode = string_to_mode(value);
 
         index = SendDlgItemMessageW(dialog, IDC_DLLS_LIST, LB_ADDSTRING, (WPARAM) -1, (LPARAM) str);
         SendDlgItemMessageW(dialog, IDC_DLLS_LIST, LB_SETITEMDATA, index, (LPARAM) dll);
 
-        HeapFree(GetProcessHeap(), 0, str);
+        free(str);
 
         count++;
     }
 
-    HeapFree(GetProcessHeap(), 0, overrides);
+    free(overrides);
 
     /* restore the previous selection, if possible  */
     if (sel >= count - 1) sel = count - 1;
@@ -594,8 +594,8 @@ static void on_remove_click(HWND dialog)
     SendMessageW(GetParent(dialog), PSM_CHANGED, 0, 0);
     set_reg_key(config_key, keypath(L"DllOverrides"), dll->name, NULL);
 
-    HeapFree(GetProcessHeap(), 0, dll->name);
-    HeapFree(GetProcessHeap(), 0, dll);
+    free(dll->name);
+    free(dll);
 
     if (SendDlgItemMessageW(dialog, IDC_DLLS_LIST, LB_GETCOUNT, 0, 0) > 0)
         SendDlgItemMessageW(dialog, IDC_DLLS_LIST, LB_SETCURSEL, max(sel - 1, 0), 0);
